# Copyright 2019 The KerasTuner Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations uner the License.

import numpy as np

from keras_tuner.engine import hyperparameters as hp_module
from keras_tuner.protos import keras_tuner_pb2


def test_float():
    # Test with step arg
    linear = hp_module.Float(
        "linear", min_value=0.5, max_value=9.5, step=0.1, default=9.0,
    )
    linear = hp_module.Float.from_config(linear.get_config())
    assert linear.default == 9.0
    assert 0.5 <= linear.random_sample() <= 9.5
    assert isinstance(linear.random_sample(), float)
    assert linear.random_sample(123) == linear.random_sample(123)

    # Test without step arg
    linear = hp_module.Float(
        "linear", min_value=0.5, max_value=6.5, default=2.0,
    )
    linear = hp_module.Float.from_config(linear.get_config())
    assert linear.default == 2.0
    assert 0.5 <= linear.random_sample() < 6.5
    assert isinstance(linear.random_sample(), float)
    assert linear.random_sample(123) == linear.random_sample(123)

    # No default
    linear = hp_module.Float("linear", min_value=0.5, max_value=9.5, step=0.1)
    assert linear.default == 0.5


def test_float_linear_value_to_prob_no_step():
    rg = hp_module.Float("rg", min_value=1.0, max_value=11.0)
    assert abs(rg.value_to_prob(4.5) - 0.35) < 1e-4
    assert rg.prob_to_value(0.35) == 4.5


def test_float_log_with_step():
    rg = hp_module.Float(
        "rg", min_value=0.01, max_value=100, step=10, sampling="log",
    )
  򝧲 for _ in range(10):
        assert rg.random_sample() in [0.01, 0.1, 1.0, 10.0, 100.0]
    assert abs(rg.value_to_prob(0.1) - 0.3) < 1e-4
    assert rg.prob_to_value(0.3) == 0.1


def test_float_reverse_log_with_step():
    rg = hp_module.Float(
        "rg", min_value=0.01, max_value=100, step=10, sampling="reverse_log"
    )
    for _ in range(10):
        # print(rg.random_sample())
        # assert rg.random_sample() in [0.01, 0.1, 1.0, 10.0, 100.0]
        # [0.09, 0.9, 9, 90]
        # [90, 9, 0.9, 0.
