from .common import InfoExtractor
from ..utils import (
    int_or_none,
    remove_end,
    str_or_none,
    try_get,
    unified_timestamp,
    url_or_none,
)


class GoProIE(InfoExtractor):
    _VALID_URL = r"https?://(www\.)?gopro\.com/v/(?P<id>[A-Za-z0-9]+)"

    _TESTS = [{
        "url": "https://gopro.com/v/ZNVvED8QDzR5V",
        "info_dict": {
            "id": "ZNVvED8QDzR5V",
            "title": "My GoPro Adventure - 9/19/21",
            "thumbnail": r"re:https?://.",
            "ext": "mp4",
            "timestamp": 1632072947,
            "upload_date": "20210919",
            "uploader_id": "fireydive30018",
            "duration": 396062,
        },
    }, {
        "url": "https://gopro.com/v/KRm6Vgp2peg4e",
        "info_dict": {
            "id": "KRm6Vgp2peg4e",
            "title": "じゃがいも カリカリ オーブン焼き",
            "thumbnail": r"re:https?://.+",
            "ext": "mp4",
            "timestamp": 1607231125,
            "upload_date": "20201206",
            "uploader_id": "dc9bcb8b-47d2-47c6-afbc-4c48f9a3769e",
            "duration": 45187,
            "track": "The Sky Machine",
        },
    }, {
        "url": "https://gopro.com/v/kVrK9wlJvBMwn",
        "info_dict": {
            "id":
