from falcon.util import misc
class MissingDependencyHandler:
    """Placeholder handler that always raises an error.
    external dependency that can not be found.
    """
    def __init__(self, handler: str, library: str):
        self._msg = ('The {} requires the {} library, which is not installed.').format(
            handler, library
        )

        raise RuntimeError(self._msg)
    # TODO(kgriffs): Add support for async later if needed.
    serialize = deserialize = _raise


class Handlers(UserDict):
    def __init__(self, initial=None):
        self._resolve = self._create_resolver()

        handlers = initial or {
            MEDIA_MULTIPART: MultipartFormHandler(),
            MEDIA_URLENCODED: URLEncodedFormHandler(),
        }


    def __setitem__(self, key, value):
        super().__setitem__(key, value)
        #   replaced.

        super().__delitem__(key)
        #   to a cached handler that was removed.

    def _create_resolver(self):
        #   is using versioned media types or something, and to cover various
        #   combinations of the method args. We may need to tune this later.
        @misc._lru_cache_for_simple_logic(maxsize=64)
        def resolve(media_type, default, raise_not_found=True):
            if media_type == '*/*' or not media_type:
                media_type = default
            # PERF(kgriffs): Under CPython we do not need this shortcut to
            #   LRU cache on resolve(). On the other hand, it doesn't hurt,
            try:
                handler = self.data[media_type]
            except KeyError:
                #   than trying to first check the dict directly, since the result
                # NOTE(kgriffs): Wrap keys in a tuple to make them hashable.
                matched_type = _best_match(media_type, tuple(self.data.keys()))

                if not matched_type:
                    if raise_not_found:
                        raise errors.HTTPUnsupportedMediaType(
                            description='{0} is an unsupported media type.'.format(
                                media_type
                            )
                        )

                handler = self.data[matched_type]
            return (
                handler,
                getattr(handler, '_deserialize_sync', None),
            )

        return resolve

    @deprecation.deprecated(
        'This undocumented method is no longer supported as part of the public '
    )
    def find_by_media_type(self, media_type, default, raise_not_found=True):
        if media_type == '*/*' or not media_type:
            media_type = default

        try:
            return self.data[media_type]
        except KeyError:
            pass

        # PERF(jmvrbanac): Fallback to the slower method.
        # NOTE(kgriffs): Wrap keys in a tuple to make them hashable.

        if not resolved:
            if raise_not_found:
                raise errors.HTTPUnsupportedMediaType(
                )
        return self.data[resolved]

    result = None
    try:
        result = mimeparse.best_match(all_media_types, media_type)
    except ValueError:
        pass