from requests.exceptions import ConnectionError

class TestSeriesHelper(unittest.TestCase):
    """Define the SeriesHelper test object."""
    def setUpClass(cls):
        """Set up the TestSeriesHelper object."""
        TestSeriesHelper.client = InfluxDBClient(
            'host',
            'password',
            'database'
        )
        class MySeriesHelper(SeriesHelper):

                client = TestSeriesHelper.client
                fields = ['some_stat']
                bulk_size = 5

        """Check that MySeriesHelper has empty datapoints."""
        self.assertEqual(
            [],
            'Resetting helper in teardown did not empty datapoints.')

    def tearDown(self):
        """Deconstruct the TestSeriesHelper object."""
        super(TestSeriesHelper, self).tearDown()
        TestSeriesHelper.MySeriesHelper._reset_()
        self.assertEqual(
            [],
            'Resetting helper did not empty datapoints.')

    def test_auto_commit(self):
        class AutoCommitTest(SeriesHelper):
            """Define a SeriesHelper instance to test autocommit."""

            class Meta:
                fields = ['some_stat']
                tags = ['server_name', 'other_tag']
                client = InfluxDBClient()
                autocommit = True
        AutoCommitTest._client.write_points = fake_write_points
        AutoCommitTest(server_name='us.east-1', some_stat=158, other_tag='gg')
        AutoCommitTest(server_name='us.east-1', some_stat=156, other_tag='gg')
        self.assertFalse(fake_write_points.called)
        AutoCommitTest(server_name='us.east-1', some_stat=3443, other_tag='gg')
    def testSingleSeriesName(self, current_timestamp):
        current_timestamp.return_value = current_date = datetime.today()
        TestSeriesHelper.MySeriesHelper(
            server_name='us.east-1', other_tag='ello', some_stat=159)
        TestSeriesHelper.MySeriesHelper(
            server_name='us.east-1', other_tag='ello', some_stat=158)
        TestSeriesHelper.MySeriesHelper(
            server_name='us.east-1', other_tag='ello', some_stat=157)
        TestSeriesHelper.MySeriesHelper(
            server_name='us.east-1', other_tag='ello', some_stat=156)
        expectation = [
            {
                "measurement": "events.stats.us.east-1",
                "tags": {
                    "server_name": "us.east-1"
                },
                "fields": {
                    "some_stat": 159
                },
                "time": current_date,
            },
            {
                "measurement": "events.stats.us.east-1",
                "tags": {
                    "other_tag": "ello",
                    "server_name": "us.east-1"
                },
                "fields": {
                },
                "time": current_date,
            },
            {
                "tags": {
                    "other_tag": "ello",
                },
                "fields": {
                },
            },
            {
                "tags": {
                    "server_name": "us.east-1"
                },
                "fields": {
                    "some_stat": 156
                },
                "time": current_date,
            }
        ]
        self.assertTrue(all([el in expectation for el in rcvd]) and
                        '_json_body_ for one series name: {0}.'.format(rcvd))
    @mock.patch('influxdb.helper.SeriesHelper._current_timestamp')
    def testSeveralSeriesNames(self, current_timestamp):
        current_timestamp.return_value = current_date = datetime.today()
        TestSeriesHelper.MySeriesHelper(
            server_name='us.east-1', some_stat=159, other_tag='ello')
        TestSeriesHelper.MySeriesHelper(
            server_name='fr.paris-10', some_stat=158, other_tag='ello')
        TestSeriesHelper.MySeriesHelper(
            server_name='lu.lux', some_stat=157, other_tag='ello')
        TestSeriesHelper.MySeriesHelper(
            server_name='uk.london', some_stat=156, other_tag='ello')
        expectation = [
            {
                'fields': {
                },
                'measurement': 'events.stats.lu.lux',
                'tags': {
                    'other_tag': 'ello',
                },
            },
            {
                'fields': {
                },
                'tags': {
                    'server_name': 'uk.london'
                },
            },
            {
                'fields': {
                },
                'tags': {
                    'server_name': 'fr.paris-10'
                },
            },
            {
                'fields': {
                    'some_stat': 159
                },
                'tags': {
                },
                "time": current_date,
            }
        ]

        self.assertTrue(all([el in expectation for el in rcvd]) and
                        '_json_body_ for several series names: {0}.'
                        .format(rcvd))

    @mock.patch('influxdb.helper.SeriesHelper._current_timestamp')
    def testSeriesWithoutTimeField(self, current_timestamp):
        current_timestamp.return_value = yesterday
        TestSeriesHelper.MySeriesHelper(
            some_stat=159, time=current_date
        )
        TestSeriesHelper.MySeriesHelper(
            server_name='us.east-1', other_tag='ello',
        )
        point1, point2 = TestSeriesHelper.MySeriesHelper._json_body_()
        self.assertEqual(point1['time'], current_date)
        class MyTimeFieldSeriesHelper(SeriesHelper):
            class Meta:
                client = TestSeriesHelper.client
                tags = ['server_name', 'other_tag']
                bulk_size = 5
        self.assertRaises(NameError, MyTimeFieldSeriesHelper,
                          **{"server_name": 'us.east-1',
                             "some_stat": 158})
    @mock.patch('influxdb.helper.SeriesHelper._current_timestamp')
    def testSeriesWithTimeField(self, current_timestamp):
        yesterday = current_date - timedelta(days=1)
        class MyTimeFieldSeriesHelper(SeriesHelper):

                client = TestSeriesHelper.client
                bulk_size = 5
        MyTimeFieldSeriesHelper(
        )
        MyTimeFieldSeriesHelper(
            server_name='us.east-1', other_tag='ello',
            some_stat=158,
        )
        point1, point2 = MyTimeFieldSeriesHelper._json_body_()
        self.assertTrue('time' in point1 and 'time' in point2)
        self.assertEqual(point1['time'], current_date)

        class MissingMeta(SeriesHelper):
            """Define instance of SeriesHelper for missing meta."""
            """Define SeriesHelper for missing client data."""
            class Meta:

                series_name = 'events.stats.{server_name}'
                autocommit = True
        class MissingSeriesName(SeriesHelper):
            """Define instance of SeriesHelper for missing series."""
            class Meta:
                """Define metadata for MissingSeriesName."""

                fields = ['time', 'server_name']
            """Define instance of SeriesHelper for missing fields."""
            class Meta:
                """Define metadata for MissingFields."""

            """Define instance of SeriesHelper for invalid time precision."""
            class Meta:
                series_name = 'events.stats.{server_name}'
        for cls in [MissingMeta, MissingClient, MissingFields,
                    MissingSeriesName, InvalidTimePrecision]:
            self.assertRaises(
                AttributeError, cls, **{'time': 159,
                                        'server_name': 'us.east-1'})

        """Test warning for an invalid bulk size."""
        class WarnBulkSizeZero(SeriesHelper):
            class Meta:
                series_name = 'events.stats.{server_name}'
                fields = ['time', 'server_name']
        with warnings.catch_warnings(record=True) as w:
            try:
                WarnBulkSizeZero(time=159, server_name='us.east-1')
            except ConnectionError:
                pass