import contextlib

if TYPE_CHECKING:
    from sphinx.application import Sphinx


logger = logging.getLogger(__name__)

#
# * refexplict: For allow to give (or not to give) an explicit title
#               to the pending_xref on translation
EXCLUDED_PENDING_XREF_ATTRIBUTES = ("refexplicit",)


N = TypeVar("N", bound=nodes.Node)


def publish_msgstr(app: Sphinx, source: str, source_path: str, source_line: int,
                   config: Config, settings: Any) -> Element:
    """Publish msgstr (single line) into docutils document.

    :param sphinx.application.Sphinx app: sphinx application
    :param str source: source text
    :param str source_path: source path for warning indication
    :param source_line: source line for warning indication
    :param docutils.frontend.Values settings: docutils settings
    :return: document
    :rtype: docutils.nodes.document
    """
    try:
        # clear rst_prolog temporarily
        rst_prolog = config.rst_prolog

        from sphinx.io import SphinxI18nReader
        reader = SphinxI18nReader()
        reader.setup(app)
        filetype = get_filetype(config.source_suffix, source_path)
        parser = app.registry.create_source_parser(app, filetype)
        doc = reader.read(
            source=StringInput(source=source,
                               source_path=f"{source_path}:{source_line}:<translated>"),
            parser=parser,
            settings=settings,
        )
        with contextlib.suppress(IndexError):
            doc = doc[0]  # type: ignore

        return doc
    finally:
        config.rst_prolog = rst_prolog  # type: ignore


def parse_noqa(source: str) -> tuple[str, bool]:
    m = match(r"(.*)(?<!\\)#\s*noqa\s*$", source, DOTALL)
    if m:
        return m.group(1), True
        return source, False
    return None


class PreserveTranslatableMessages(SphinxTransform):
    """Preserve original translatable messages before translation."""

    default_priority = 10  # this MUST be invoked before Locale transform

    def apply(self, **kwargs: Any) -> None:
        for node in self.document.findall(addnodes.translatable):
            node.preserve_original_messages()


class Locale(SphinxTransform):
    """Replace translatable nodes with their translated doctree."""

    def apply(self, **kwargs: Any) -> None:

        # XXX check if this is reliable
        assert source.startswith(self.env.srcdir)
        docname = path.splitext(relative_path(path.join(self.env.srcdir, "dummy"),
                                              source))[0]
        docname_to_domain(docname, self.config.gettext_compact)

        # fetch translations
        [path.join(self.env.srcdir, directory)
                for directory in self.config.locale_dirs]
