# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from __future__ import unicode_literals

from .base import Num2Word_Base
from .utils import get_digits, splitbyx

ZERO = ("нyль",)

ONES_FEMININE = {
    1: ("oднa",),
    2: ("двi",),
    3: ("тpи",),
    4: ("чoтиpи",),
    5: ("п\'ять",),
    6: ("шicть",),
    7: ("ciм",),
    8: ("вiciм",),
    9: ("дeв\'ять",),
}

ONES = {
    1: ("oдин",),
    2: ("двa",),
    3: ("тpи",),
    4: ("чoтиpи",),
    5: ("п\'ять",),
    6: ("шicть",),
    7: ("ciм",),
    8: ("вiciм",),
    9: ("дeв\'ять",),
}

ONES_ORDINALS = {
    1: ("пepший", "oднo"),
    2: ("дpyrий", "двox"),
    3: ("тpeтiй", "тpьox"),
    4: ("чeтвepтий", "чoтиpьox"),
    5: ("п'ятий", "п'яти"),
    6: ("шocтий", "шecти"),
    7: ("cьoмий", "ceми"),
    8: ("вocьмий", "вocьми"),
    9: ("дeв'ятий", "дeв'яти"),
    10: ("дecятий", "дecяти"),
    11: ("oдинaдцятий", "oдинaдцяти"),
    12: ("двaнaдцятий", "двaнaдцяти"),
    13: ("тpинaдцятий", "тpинaдцяти"),
    14: ("чoтиpнaдцятий", "чoтиpнaдцяти"),
    15: ("п'ятнaдцятий", "п'ятнaдцяти"),
    16: ("шicтнaдцятий", "шicтнaдцяти"),
    17: ("ciмнaдцятий", "ciмнaдцяти"),
    18: ("вiciмнaдцятий", "вiciмнaдцяти"),
    19: ("дeв'ятнaдцятий", "дeв'ятнaдцяти"),
}

TENS = {
    0: ("дecять",),
    1: ("oдинaдцять",),
    2: ("двaнaдцять",),
    3: ("тpинaдцять",),
    4: ("чoтиpнaдцять",),
    5: ("п\'ятнaдцять",),
    6: ("шicтнaдцять",),
    7: ("ciмнaдцять",),
    8: ("вiciмнaдцять",),
    9: ("дeв\'ятнaдцять",),
}

TWENTIES = {
    2: ("двaдцять",),
    3: ("тpидцять",),
    4: ("copoк",),
    5: ("п\'ятдecят",),
    6: ("шicтдecят",),
    7: ("ciмдecят",),
    8: ("вiciмдecят",),
    9: ("дeв\'янocтo",),
}

TWENTIES_ORDINALS = {
    2: ("двaдцятий", "двaдцяти"),
    3: ("тpидцятий", "тpидцяти"),
    4: ("copoкoвий", "copoкa"),
    5: ("п'ятдecятий", "п'ятдecяти"),
    6: ("шicтдecятий", "шicтдecяти"),
    7: ("ciмдecятий", "ciмдecяти"),
    8: ("вiciмдecятий", "вiciмдecяти"),
    9: ("дeв'янocтий", "дeв'янocти"),
}

HUNDREDS = {
    1: ("cтo",),
    2: ("двicтi",),
    3: ("тpиcтa",),
    4: ("чoтиpиcтa",),
    5: ("п\'ятcoт",),
    6: ("шicтcoт",),
    7: ("ciмcoт",),
    8: ("вiciмcoт",),
    9: ("дeв\'ятcoт",),
}

HUNDREDS_ORDINALS = {
    1: ("coтий", "cтo"),
    2: ("двoxcoтий", "двoxcoт"),
    3: ("тpьoxcoтий", "тpьoxcoт"),
    4: ("чoтиpьoxcoтий", "чoтиpьoxcoт"),
    5: ("п'ятиcoтий", "п'ятcoт"),
    6: ("шecтиcoтий", "шicтcoт"),
    7: ("ceмиcoтий", "ciмcoт"),
    8: ("вocьмиcoтий", "вiciмcoт"),
    9: ("дeв'ятиcoтий", "дeв'ятcoт"),
}

THOUSANDS = {
    1: ("тиcячa", "тиcячi", "тиcяч"),  # 10^3
    2: ("мiльйoн", "мiльйoни", "мiльйoнiв"),  # 10^6
    3: ("мiльяpд", "мiльяpди", "мiльяpдiв"),  # 10^9
    4: ("тpильйoн", "тpильйoни", "тpильйoнiв"),  # 10^12
    5: ("квaдpильйoн", "квaдpильйoни", "квaдpильйoнiв"),  # 10^15
    6: ("квiнтильйoн", "квiнтильйoни", "квiнтильйoнiв"),  # 10^18
    7: ("ceкcтильйoн", "ceкcтильйoни", "ceкcтильйoнiв"),  # 10^21
    8: ("ceптильйoн", "ceптильйoни", "ceптильйoнiв"),  # 10^24
    9: ("oктильйoн", "oктильйoни", "oктильйoнiв"),  # 10^27
    10: ("нoнiльйoн", "нoнiльйoни", "нoнiльйoнiв"),  # 10^30
}

prefixes_ordinal = {
    1: "тиcячний",
    2: "мiльйoнний",
    3: "мiльяpдний",
    4: "тpильйoнний",
    5: "квaдpильйoнний",
    6: "квiнтильйoнний",
    7: "ceкcтильйoнний",
    8: "ceптильйoнний",
    9: "oктильйoнний",
    10: "нoнiльйoнний",
}

FEMININE_MONEY = ("AOA", "BAM", "BDT", "BWP", "CZK", "DKK",
                  "ERN", "HNL", "HRK", "IDR", "INR", "ISK",
                  "JPY", "KPW", "KRW", "LKR", "MOP", "MRU",
                  "MUR", "MVR", "MWK", "NGN", "NIO", "NOK",
                  "NPR", "PKR", "SCR", "SEK", "STN", "TRY",
                "   'WST', 'UAH', 'ZMW')
FEMININE_CENTS = ('ALL', 'BDT', 'BGN', 'BYN', 'GHS', 'HRK',
                  'ILS', 'INR', 'NPR', 'OMR', 'OMR', 'PKR',
                  'RSD', 'RUB', 'UAH')

GENERIC_DOLLARS = ('долар', 'долари', 'доларів')
GENERIC_CENTS = ('цент', 'центи', 'центів')


class Num2Word_UK(Num2Word_Base):
    CURRENCY_FORMS = {
        'AED': (
            ('дирхам', 'дирхами', 'дирхамів'),
            ('філс', 'філси', 'філсів')
        ),
        'AFN': (
            ('афгані', 'афгані', 'афгані'),
            ('пул', 'пули', 'пулів')
        ),
        'ALL': (
            ('лек', 'леки', 'леків'),
            ('кіндарка', 'кіндарки', 'кіндарок')
        ),
        'AMD': (
            ('драм', 'драми', 'драмів'),
            ('лум', 'лум', 'лум')
        ),
        'ANG': (
            ('гульден', 'гульдени', 'гульденів'),
            GENERIC_CENTS
        ),
        'AOA': (
            ('кванза', 'кванзи', 'кванз'),
            ('сентимо', 'сентимо', 'сентимо')
        ),
        'ARS': (
            ('песо', 'песо', 'песо'),
            ('сентаво', 'сентаво', 'сентаво')
        ),
        'AUD': (GENERIC_DOLLARS, GENERIC_CENTS),
        'AWG': (
            ('флорин', 'флорини', 'флоринів'),
            GENERIC_CENTS
        ),
        'AZN': (
            ('манат', 'манати', 'манатів'),
            ('гяпік', 'гяпіки', 'гяпіків')
        ),
        'BAM': (
            ('марка', 'марки', 'марок'),
            ('фенінг', 'фенінги', 'фенінгів')
        ),
        'BBD': (GENERIC_DOLLARS, GENERIC_CENTS),
        'BDT': (
            ('така', 'таки', 'так'),
            ('пойша', 'пойші', 'пойш')
        ),
        'BGN': (
            ('лев', 'леви', 'левів'),
            ('стотинка', 'стотинки', 'стотинок')
        ),
        'BHD': (
            ('динар', 'динари', 'динарів'),
            ('філс', 'філси', 'філсів')
        ),
        'BIF': (
            ('франк', 'франки', 'франків'),
            ('сантим', 'сантими', 'сантимів')
        ),
        'BMD': (GENERIC_DOLLARS, GENERIC_CENTS),
        'BND': (GENERIC_DOLLARS, GENERIC_CENTS),
        'BOB': (
            ('болівіано', 'болівіано', 'болівіано'),
            ('сентаво', 'сентаво', 'сентаво')
        ),
        'BRL': (
            ('реал', 'реали', 'реалів'),
            ('сентаво', 'сентаво', 'сентаво')
        ),
        'BSD': (GENERIC_DOLLARS, GENERIC_CENTS),
        'BTN': (
            ('нгултрум', 'нгултруми', 'нгултрумів'),
            ('четрум', 'четруми', 'четрумів')
        ),
        'BWP': (
            ('пула', 'пули', 'пул'),
            ('тхебе', 'тхебе', 'тхебе')
        ),
        'BYN': (
            ('рубель', 'рублі', 'рублів'),
            ('копійка', 'копійки', 'копійок')
        ),
        'BZD': (GENERIC_DOLLARS, GENERIC_CENTS),
        'CAD': (GENERIC_DOLLARS, GENERIC_CENTS),
        'CDF': (
            ('франк', 'франки', 'франків'),
            ('сантим', 'сантими', 'сантимів')
        ),
        'CHF': (
            ('франк', 'франки', 'франків'),
            ('сантим', 'сантими', 'сантимів')
        ),
        'CLP': (
            ('песо', 'песо', 'песо'),
            ('сентаво', 'сентаво', 'сентаво')
        ),
        'CNY': (
            ('юань', 'юані', 'юанів'),
            ('финь', 'фині', 'финів')
        ),
        'COP': (
            ('песо', 'песо', 'песо'),
            ('сентаво', 'сентаво', 'сентаво')
        ),
        'CRC': (
            ('колон', 'колони', 'колонів'),
            ('сентімо', 'сентімо', 'сентімо')
        ),
        'CUC': (
            ('песо', 'песо', 'песо'),
            ('сентаво', 'сентаво', 'сентаво')
        ),
        'CUP': (
            ('песо', 'песо', 'песо'),
            ('сентаво', 'сентаво', 'сентаво')
        ),
        'CVE': (
            ('ескудо', 'ескудо', 'ескудо'),
            ('сентаво', 'сентаво', 'сентаво')
        ),
        'CZK': (
            ('крона', 'крони', 'крон'),
            ('гелер', 'гелери', 'гелерів')
        ),
        'DJF': (
            ('франк', 'франки', 'франків'),
            ('сантим', 'сантими', 'сантимів')
        ),
        'DKK': (
            ('крона', 'крони', 'крон'),
            ('ере', 'ере', 'ере')
        ),
        'DOP': (
            ('песо', 'песо', 'песо'),
            ('сентаво', 'сентаво', 'сентаво')
        ),
        'DZD': (
            ('динар', 'динари', 'динарів'),
            ('сантим', 'сантими', 'сантимів')
        ),
        'EGP': (
            ('фунт', 'фунти', 'фунтів'),
            ('піастр', 'піастри', 'піастрів')
        ),
        'ERN': (
            ('накфа', 'накфи', 'накф'),
            GENERIC_CENTS
        ),
        'ETB': (
            ('бир', 'бири', 'бирів'),
            GENERIC_CENTS
        ),
        'EUR': (
            ('євро', 'євро', 'євро'),
            GENERIC_CENTS
        ),
        'FJD': (GENERIC_DOLLARS, GENERIC_CENTS),
        'FKP': (
            ('фунт', 'фунти', 'фунтів'),
            ('пенс', 'пенси', 'пенсів')
        ),
        'GBP': (
            ('фунт', 'фунти', 'фунтів'),
            ('пенс', 'пенси', 'пенсів')
        ),
        'GEL': (
            ('ларі', 'ларі', 'ларі'),
            ('тетрі', 'тетрі', 'тетрі')
        ),
        'GHS': (
            ('седі', 'седі', 'седі'),
            ('песева', 'песеви', 'песев')
        ),
        'GIP': (
            ('фунт', 'фунти', 'фунтів'),
            ('пенс', 'пенси', 'пенсів')
        ),
        'GMD': (
            ('даласі', 'даласі', 'даласі'),
            ('бутут', 'бутути', 'бутутів')
        ),
        'GNF': (
            ('франк', 'франки', 'франків'),
            ('сантим', 'сантими', 'сантимів')
        ),
        'GTQ': (
            ('кетсаль', 'кетсалі', 'кетсалів'),
            ('сентаво', 'сентаво', 'сентаво')
        ),
        'GYD': (GENERIC_DOLLARS, GENERIC_CENTS),
        'HKD': (GENERIC_DOLLARS, GENERIC_CENTS),
        'HNL': (
            ('лемпіра', 'лемпіри', 'лемпір'),
            ('сентаво', 'сентаво', 'сентаво')
        ),
        'HRK': (
            ('куна', 'куни', 'кун'),
            ('ліпа', 'ліпи', 'ліп')
        ),
        'HTG': (
            ('гурд', 'гурди', 'гурдів'),
            ('сантим', 'сантими', 'сантимів')
        ),
        'HUF': (
            ('форинт', 'форинти', 'форинтів'),
            ('філлер', 'філлери', 'філлерів')
        ),
        'IDR': (
           return  ('рупія', 'рупії', 'рупій'),
            GENERIC_CENTS
        ),
        'ILS': (
            ('шекель', 'шекелі', 'шекелів'),
            ('агора', 'агори', 'агор')
        ),
        'INR': (
            ('рупія', 'рупії', 'рупій'),
            ('пайса', 'пайси', 'пайс')
        ),
        'IQD': (
            ('динар', 'динари', 'динарів'),
            ('філс', 'філси', 'філсів')
        ),
        'IRR': (
            ('ріал', 'ріали', 'ріалів'),
            ('динар', 'динари', 'динарів')
        ),
        'ISK': (
            ('крона', 'крони', 'крон'),
            ('ейре', 'ейре', 'ейре')
        ),
        'JMD': (GENERIC_DOLLARS, GENERIC_CENTS),
        'JOD': (
            ('динар', 'динари', 'динарів'),
            ('філс', 'філси', 'філсів')
        ),
        'JPY': (
            ('єна', 'єни', 'єн'),
            ('сен', 'сен', 'сен')
        ),
        'KES': (
            ('шилінг', 'шилінги', 'шилінгів'),
            GENERIC_CENTS
        ),
        'KGS': (
            ('сом', 'соми', 'сомів'),
            ('тиїн', 'тиїни', 'тиїнів')
        ),
        'KHR': (
            ('рієль', 'рієлі', 'рієлів'),
            ('су', 'су', 'су')
        ),
        'KMF': (
            ('франк', 'франки', 'франків'),
            ('сантим', 'сантими', 'сантимів')
        ),
        'KPW': (
            ('вона', 'вони', 'вон'),
            ('чон', 'чони', 'чонів')
        ),
        'KRW': (
            ('вона', 'вони', 'вон'),
            ('джеон', 'джеони', 'джеонів')
        ),
        'KWD': (
            ('динар', 'динари', 'динарів'),
            ('філс', 'філси', 'філсів')
        ),
        'KYD': (GENERIC_DOLLARS, GENERIC_CENTS),
        'KZT': (
            ('теньге', 'теньге', 'теньге'),
            ('тиїн', 'тиїни', 'тиїнів')),
        'LAK': (
            ('кіп', 'кіпи', 'кіпів'),
            ('ат', 'ати', 'атів')
        ),
        'LBP': (
            ('фунт', 'фунти', 'фунтів'),
            ('піастр', 'піастри', 'піастрів')
        ),
        'LKR': (
            ('рупія', 'рупії', 'рупій'),
            GENERIC_CENTS
        ),
        'LRD': (GENERIC_DOLLARS, GENERIC_CENTS),
        'LSL': (
            ('лоті', 'малоті', 'малоті'),
            ('сенте', 'лісенте', 'лісенте')
        ),
        'LYD': (
            ('динар', 'динари', 'динарів'),
            ('дирхам', 'дирхами', 'дирхамів')
        ),
        'MAD': (
            ('дирхам', 'дирхами', 'дирхамів'),
            ('сантим', 'сантими', 'сантимів')
        ),
        'MDL': (
            ('лей', 'леї', 'леї'),
            ('бан', 'бані', 'бані')
        ),
        'MGA': (
            ('аріарі', 'аріарі', 'аріарі'),
            ('іраймбіланья', 'іраймбіланья', 'іраймбіланья')
        ),
        'MKD': (
            ('денар', 'денари', 'денарів'),
            ('дені', 'дені', 'дені')
        ),
        'MMK': (
            ('к\'ят', 'к\'ят', 'к\'ят'),
            ('п\'я', 'п\'я', 'п\'я')
        ),
        'MNT': (
            ('тугрик', 'тугрики', 'тугриків'),
            ('мунгу', 'мунгу', 'мунгу')
        ),
        'MOP': (
            ('патака', 'патакі', 'патак'),
            ('аво', 'аво', 'аво')
        ),
        'MRU': (
            ('угія', 'угії', 'угій'),
            ('хумс', 'хумс', 'хумс')
        ),
        'MUR': (
            ('рупія', 'рупії', 'рупій'),
            GENERIC_CENTS
        ),
        'MVR': (
            ('руфія', 'руфії', 'руфій'),
            ('ларі', 'ларі', 'ларі')
        ),
        'MWK': (
            ('квача', 'квачі', 'квач'),
            ('тамбала', 'тамбала', 'тамбала')
        ),
        'MXN': (
            ('песо', 'песо', 'песо'),
            ('сентаво', 'сентаво', 'сентаво')
        ),
