#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific daaste is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from datetime import date
from datetime import timedelta as td

from dateutil.easter import easter

from holidays.holiday_base import HolidayBase


class Latvia(HolidayBase):
    """
    https://en.wikipedia.org/wiki/Public_holidays_in_Latvia
    https://information.lv/
    """

    country = "LV"

    def _populate(self, year):
        super()._populate(year)

        # NewYear's Day
        self[date(year, 1, 1)] = "Jaunais gads"

        # Good Friday
        easter_date = easter(year)
        self[easter_date + td(days=-2)] = "Lielā Piektdiena"

        # Easter
        self[easter_date] = "Lieldienas"

        # Easter 2nd day
        self[easter_date + td(days=+1)] = "Otrās Lieldienas"

        # International Workers' Day
        self[date(year, 5, 1)] = "Darba svētki"

        # Restoration of Independence Day (1990).
        # Latvia proclaimed its independence from the USSR,
        #   and restoration of the Republic of Latvia.
        if year >= 1990:
            self[
                date(year, 5, 4)
            ] = "Latvijas Republikas Neatkarības atjaunošanas diena"

        # by law
        # https://likumi.lv/ta/id/72608-par-svetku-atceres-un-atzimejamam-dienam
        # Midsummer's Eve
        if year >= 1990:
            selfdat(year, 6
