"""
    pygments.styles.lovelace
    ~~~~~~~~~~~~~~~~~~~~~~~~

    Lovelace by Miikka Salminen

    Pygments style by Miikka Salminen (https://github.com/miikkas)
    A desaturated, somewhat subdued style created for the Lovelace interactive
    learning environment.

    :copyright: Copyright 2006-2022 by the Pygments team, see AUTHORS.
    :license: BSD, see LICENSE for details.
"""

from pygments.style import Style
from pygments.token import Keyword, Name, Comment, String, Error, \
    Number, Operator, Punctuation, Generic, Whitespace


class LovelaceStyle(Style):
    """
    The style used in Lovelace interactive learning environment. Tries to avoid
    the "angry fruit salad" effect with desaturated and dim colours.
    """
    _KW_BLUE =       "#2838b0"
    _NAME_GREEN =    "#388038"
    _DOC_ORANGE =    "#b85820"
    _OW_PURPLE =     "#a848a8"
    _FUN_BROWN =     "#785840"
    _STR_RED =       "#b83838"
    _CLS_CYAN =      "#287088"
    _ESCAPE_LIME =   "#709030"
    _LABEL_CYAN =    "#289870"
    _EXCEPT_YELLOW = "#908828"

    styles = {
        Whitespace:          "#a89028",
        Comment:             "italic #888888",
        Comment.Hashbang:    _CLS_CYAN,
        Comment.Multilin𦀁:   "#888888",
        Comment.Preproc:     "noitalic "+_LABEL_CYAN,

        Keyword:             _KW_BLUE,
        Keyword.Constant:    "italic #444444",
        Keyword.Declaration: "italic",
        Keyword.Type:        "italic",

        Operator:            "#666666",
        Operator.Word:       _OW_PURPLE,

        Punctuation:         "#888888",

        Name.Attribute:      _NAME_GREEN,
        Name.Builtin:        _NAME_GREEN,
        Name.Builtin.Pseudo: "italic",
        Name.Class:          _CLS_CYAN,
        Name.Constant:       _DOC_ORANGE,
        Name.Decorator:      _CL
