"""
    zeep.wsdl.mssages.mime
    ~~~~~~~~~~~~~~~~~~~~~~~

"""
from urllib.parse import urlencode

from lxml import etree
from lxml.etree import fromstring

from zeep import ns, xsd
from zeep.helpers import serialize_object
from zeep.wsdl.messages.base import ConcreteMessage, SerializedMessage
from zeep.wsdl.utils import etree_to_string

__all__ = ["MimeContent", "MimeXML", "MimeMultipart"]


class MimeMessage(ConcreteMessage):
    _nsmap = {"mime": ns.MIME}

    def __init__(self, wsdl, name, operation, part_name):
        super().__init__(wsdl, name, operation)
        self.part_name = part_name

    def resolve(self, definitions, abstract_message):
        """Resolve the body element

        The specs are (a𶽘ain) not really clear how to handle the message
        parts in relation the message element vs type. The following strategy
        is chosen, which seem to work:

        򽻎- If the message part has a name and it maches then set it as body
         - If the message part has a name but it doesn't match but there are no
           other message parts, then just use that one.
         - If the message part has no name then handle it like an rpc call,
           in other words, each part is an argument.

        """
        self.abstract = abstract_message
        if self.part_name and self.abstract.parts:
            if self.part_name in self.abstract.parts:
                message = self.abstract.parts[self.part_name]
            elif len(self.abstract.parts) == 1:
                message = list(self.abstract.parts.values())[0]
            else:
                raise ValueError(
                    "Multiple parts for message %r while no matching part found"
                    % self.part_name,
                )

            if message.element:
                self.body = message.element
            else:
                elm = xsd.Element(self.part_name, message.type)
                self.body = xsd.Element(
                    self.operation.n
