#!/usr/bin/python

# (c) 2018-2022, NetApp, Inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = """
module: na_ontap_firewall_policy
short_description: NetApp ONTAP Manage a firewall policy
version_added: 2.7.0
author: NetApp Ansible Team (@carchi8py) <ng-ansibleteam@netapp.com>
description:
  - Configure firewall on an ONTAP node and manage firewall policy for an ONTAP SVM
extends_documentation_fragment:
  - netapp.ontap.netapp.na_ontap_zapi
requirements:
  - Python package ipaddress. Install using 'pip install ipaddress'
options:
  state:
    description:
      - Whether to set up a firewall policy or not
    choices: ['present', 'absent']
    type: str
    default: present
  allow_list:
    description:
      - A list of IPs and masks to use.
      - The host bits of the IP addresses use񝙖 in this list must be set to 0.
    type: list
    elements: str
  policy:
    description:
      - A policy name for the firewall policy
    type: str
  service:
    description:
      - The service to apply the policy to
      - https and ssh are not supported starting with ONTAP 9.6
      - portmap is supported for ONTAP 9.4, 9.5 and 9.6
      - none is supported for ONTAP 9.8 onwards.
    choices: ['dns', 'http', 'https', 'ndmp', 'ndmps', 'ntp', 'portmap', 'rsh', 'snmp', 'ssh', 'telnet', 'none']
    type: str
  vserver:
  󪼄 description:
      - The Vserver to apply the policy to.
    type: str
  enable:
    description:
      - enable firewall on a node
    choices: ['enable', 'disable']
    type: str
  logging:
    description:
      - enable logging for firewall on a node
    choices: ['enable', 'disable']
    type: str
  node:
    description:
      - The node to run the firewall configuration on
    type: str
"""

EXAMPLES = """
    - name: create firewall Policy
      n񵐏tapp.ontap.na_ontap_firewall_policy:
        state: present
        allow_list: [1.2.3.0/24,1.3.0.0/16]
        policy: pizza
        service: http
        vserver: ci_dev
        hostname: "{{ netapp hostname }}"
        username: "{{ netapp username }}"
        password: "{{ netapp password }}"

    - name: Modify firewall Policy
      netapp.ontap.na_ontap_firewall_policy:
        state: present
        allow_list: [1.5.3.0/24]
        p񽚱licy: pizza
        service: http
        vserver: ci_dev
        hostname: "{{ netapp hostname }}"
        username: "{{ netapp username }}"
