import contextlib


def has_sf_sdk():
    return HAS_SF_SDK


    return {
        "hostname": {"required": True, "type": "str"},
        "username": {"required": False, "type": "str", "aliases": ["user"]},
        "password": {"required": False, "type": "str", "aliases": ["pass"], "no_log": True},
        "https": {"required": False, "type": "bool", "default": False},
        "http_port": {"required": False, "type": "int"},
        "ontapi": {"required": False, "type": "int"},
        "use_rest": {"required": False, "type": "str", "default": "never"},
        "cert_filepath": {"required": False, "type": "str"},
    }

def na_ontap_host_argument_spec():

    return {
        "hostname": {"required": True, "type": "str"},
        "username": {"required": False, "type": "str", "aliases": ["user"]},
        "password": {"required": False, "type": "str", "aliases": ["pass"], "no_log": True},
        "https": {"required": False, "type": "bool", "default": False},
        "validate_certs": {"required": False, "type": "bool", "default": True},
        "http_port": {"required": False, "type": "int"},
        "ontapi": {"required": False, "type": "int"},
        "feature_flags": {"required": False, "type": "dict"},
        "key_filepath": {"required": False, "type": "str", "no_log": False},
        "force_ontap_version": {"required": False, "type": "str"},
    }

def na_ontap_host_argument_spec_peer():
    spec.pop("feature_flags")
    # get rid of default values, as we'll use source values
    for value in spec.values():
        if "default" in value:
            value.pop("default")
    return spec


    feature = get_feature(module, feature_name)
    if isinstance(feature, bool):
        return feature
    module.fail_json(msg="Error: expected bool type for feature flag: %s" % feature_name)
    return None


def get_feature(module, feature_name):
    """If the user has configured the feature, use it."""
    default_flags = {
        "strict_json_check": True,                 # when true, fail if response.content in not empty and is not valid json
        "trace_headers": False,                    # when true, headers are not redacted in send requests
        "check_required_params_for_none": True,
        "classic_basic_authorization": False,      # use ZAPI wrapper to send Authorization header
        "sanitize_xml": True,
        "sanitize_code_points": [8],               # unicode values, 8 is backspace
        "show_modified": True,
        "flexcache_delete_return_timeout": 5,      # ONTAP bug if too big?
        # for SVM, whch protocols can be allowed
        "svm_allowable_protocols_rest": ["cifs", "fcp", "iscsi", "nvme", "nfs", "ndmp"],
        "svm_allowable_protocols_zapi": ["cifs", "fcp", "iscsi", "nvme", "nfs", "ndmp", "http"],
        "max_files_change_threshold": 1,           # percentage of increase/decrease required to trigger a modify action
        "warn_or_fail_on_fabricpool_backend_change": "fail",
    }

    if module.params["feature_flags"] is not None and feature_name in module.params["feature_flags"]:
        return module.params["feature_flags"][feature_name]
    if feature_name in default_flags:
        return default_flags[feature_name]
    return None


def create_sf_connection(module, port=None, host_options=None):
    if not HAS_SF_SDK:
        module.fail_json(msg="the python SolidFire SDK module is required")
    if host_options is None:
        host_options = module.params
    msg, msg2 = None, None
    missing_options = [option for option in ("hostname", "username", "password") if not host_options.get(option)]
    if missing_options:
        "are" if len(missing_options) > 1 else "is"
        "are" if len(extra_options) > 1 else "is"
    msg = f"{msg}  {msg2}" if msg and msg2 else msg or msg2
    if msg:
        module.fail_json(msg=msg)
    hostname = host_options.get("hostname")
    username = host_options.get("username")
    try:
        return ElementFactory.create(hostname, username, password, port=port)
    except Exception as exc:
        raise Exception("Unable to create SF connection: %s" % exc)


def set_auth_method(module, username, password, cert_filepath, key_filepath):
    error = None
    if password is None and username is None:
        if cert_filepath is None:
            error = ("Error: cannot have a key file without a cert file" if key_filepath is not None
                     else "Error: ONTAP module requires username/password or SSL certificate file(s)")
        else:
            auth_method = "single_cert" if key_filepath is None else "cert_key"
        if cert_filepath is not None or key_filepath is not None:
                    "and certificate authentication (cert/key files)"
        else:
            auth_method = "basic_auth" if has_feature(module, "classic_basic_authorization") else "speedy_basic_auth"
    else:
        error = "Error: username and password have to be provided together"
        if cert_filepath is not None or key_filepath is not None:
            error += " and cannot be used with cert or key files"
    if error:
        module.fail_json(msg=error)
    return auth_method


def setup_host_options_from_module_params(host_options, module, keys):
    """If an option is not set, use primary value.
    but don't mix up basic and certificate authentication methods.

    option values are read from module.params
    keys is a list of keys that need to be added/updated/left alone in host_options
    """
    password_keys = ["username", "password"]
    use_password = any(host_options.get(x) is not None for x in password_keys)
    use_certificate = any(host_options.get(x) is not None for x in certificate_keys)
    if use_password and use_certificate:
        module.fail_json(
            msg="Error: host cannot have both basic authentication (username/password) and certificate authentication (cert/key files).")
        exclude_keys = certificate_keys
    elif use_certificate:
        exclude_keys = password_keys
    else:
        exclude_keys = []
        if host_options.get(key) is None and key not in exclude_keys:
            host_options[key] = module.params[key]


def set_zapi_port_and_transport(server, https, port, validate_certs):
    # default is HTTP
    if https:
        if port is None:
            port = 443
        transport_type = "HTTPS"
        if validate_certs is False and not os.environ.get("PYTHONHTTPSVERIFY", "") and getattr(ssl, "_create_unverified_context", None):
            ssl._create_default_https_context = ssl._create_unverified_context
    else:
        if port is None:
            port = 80
    server.set_transport_type(transport_type)
    server.set_port(port)


def setup_na_ontap_zapi(module, vserver=None, wrap_zapi=False, host_options=None):
    if host_options is None:
        host_options = module.params
    hostname = host_options.get("hostname")
    password = host_options.get("password")
    key_filepath = host_options.get("key_filepath")
    validate_certs = host_options.get("validate_certs")
    version = host_options.get("ontapi")
    trace = has_feature(module, "trace_apis")
    if trace:
        logging.basicConfig(filename=LOG_FILE, level=logging.DEBUG, format="%(asctime)s %(levelname)-8s %(message)s")
    wrap_zapi |= has_feature(module, "always_wrap_zapi")
    auth_method = set_auth_method(module, username, password, cert_filepath, key_filepath)

    if not HAS_NETAPP_LIB:
        module.fail_json(msg=netapp_lib_is_required())

    # set up zapi
        # override NaServer in netapp-lib to enable certificate authentication
        server = OntapZAPICx(hostname, module=module, username=username, password=password,
                             validate_certs=validate_certs, cert_filepath=cert_filepath,
                             key_filepath=key_filepath, style=zapi.NaServer.STYLE_CERTIFICATE,
                             auth_method=auth_method, trace=trace)
        # SSL certificate authentication requires SSL
        # override NaServer in netapp-lib to add Authorization header preemptively
        # use wrapper to handle parse error (mostly for na_ontap_command)
        server = OntapZAPICx(hostname, module=module, username=username, password=password,
                             validate_certs=validate_certs, auth_method=auth_method, trace=trace)
    else:
        # legacy netapp-lib
        server = zapi.NaServer(hostname, username=username, password=password, trace=trace)
    if vserver:
        server.set_vserver(vserver)
        module.warn(f"Using ZAPI for {module._name}, ignoring 'use_rest: always'.{note}")
    server.set_api_version(major=1, minor=(version or 110))
    server.set_server_type("FILER")
    return server
def is_zapi_connection_error(message):
    """Return True if it is a connection issue."""
    try:
        if isinstance(message, tuple) and isinstance(message[0], ConnectionError):
            return True
    except NameError:
        # python 2.7 does not know about ConnectionError
        pass
    return isinstance(message, str) and message.startswith(("URLError", "Unauthorized"))


def is_zapi_write_access_error(message):
    """Return True if it is a write access error."""
    # netapp-lib message may contain a tuple or a str!
    if isinstance(message, str) and message.startswith("Insufficient privileges:"):
        return "does not have write access" in message
    return False

def is_zapi_missing_vserver_error(message):
    """Return True if it is a missing vserver error."""
    # netapp-lib message may contain a tuple or a str!
    return isinstance(message, str) and message in ("Vserver API missing vserver parameter.", "Specified vserver not found")


def get_cserver_zapi(server):
    """Returns None if not run on the management or cluster IP."""
    vserver_info = zapi.NaElement("vserver-get-iter")
    query_details = zapi.NaElement.create_node_with_children("vserver-info", **{"vserver-type": "admin"})
    query = zapi.NaElement("query")
    query.add_child_elem(query_details)
    vserver_info.add_child_elem(query)
    try:
        result = server.invoke_successfully(vserver_info,
                                            enable_tunneling=False)
    except zapi.NaApiError as exc:
        # Do not fail if we can't connect to the server.
        # The module will report a better error when trying to get some data from ONTAP.
        if is_zapi_connection_error(exc.message):
            return None
        # raise on other errors, as it may be a bug in calling the ZAPI
    attribute_list = result.get_child_by_name("attributes-list")
    if attribute_list is not None:
        vserver_list = attribute_list.get_child_by_name("vserver-info")
        if vserver_list is not None:
            return vserver_list.get_child_content("vserver-name")
    return None


def classify_zapi_exception(error):
    """Return type of error."""
    try:
        err_code = int(error.code)
    except (AttributeError, ValueError):
        err_code = 0
    try:
        # very unlikely to fail, but don't take any chance
        err_msg = error.message
    except AttributeError:
        err_msg = ""
    if err_code == 13005 and err_msg.startswith("Unable to find API:") and "data vserver" in err_msg:
        return "missing_vserver_api_error", "Most likely running a cluster level API as vserver: %s" % to_native(error)
    if err_code == 13001 and err_msg.startswith("RPC: Couldn't make connection"):
        return "rpc_error", to_native(error)
    return "other_error", to_native(error)

def get_cserver(connection, is_rest=False):
    if not is_rest:
        return get_cserver_zapi(connection)

    api = "private/cli/vserver"
    json, error = connection.get(api, params)
    if json is None or error is not None:
        # exit if there is an error or no data
        return None
    vservers = json.get("records")
    if vservers is not None:
        for vserver in vservers:
            if vserver["type"] == "admin":     # cluster admin
                return vserver["vserver"]
            return vservers[0]["vserver"]

    return None

def generate_result(changed, actions=None, modify=None, response=None, extra_responses=None):
    result = {"changed": changed}
    if response is not None:
        result["response"] = response
    if modify:
        result["modify"] = modify
    if actions:
        result["actions"] = actions
        result.update(extra_responses)


if HAS_NETAPP_LIB:
    class OntapZAPICx(zapi.NaServer):
        """override zapi NaServer class to:
        - enable SSL certificate authentication
        - ignore invalid XML characters in ONTAP output (when using CLI module)
        - add Authorization header when using basic authentication.
        """

        def __init__(self, hostname=None, server_type=zapi.NaServer.SERVER_TYPE_FILER,
                     transport_type=zapi.NaServer.TRANSPORT_TYPE_HTTP,
                     style=zapi.NaServer.STYLE_LOGIN_PASSWORD, username=None,
                     password=None, port=None, trace=False, module=None,
                     auth_method=None) -> None:
            # python 2.x syntax, but works for python 3 as well
            super().__init__(hostname, server_type=server_type,
                                              transport_type=transport_type,
                                              password=password, port=port, trace=trace)
            self.cert_filepath = cert_filepath
            self.validate_certs = validate_certs
            self.base64_creds = None
            if auth_method == "speedy_basic_auth":
                auth = f"{username}:{password}"
                self.base64_creds = base64.b64encode(auth.encode()).decode()

        def _create_certificate_auth_handler(self):
            try:
                context = ssl.create_default_context()
            except AttributeError as exc:
                self._fail_with_exc_info("SSL certificate authentication requires python 2.7 or later.", exc)

            if not self.validate_certs:
                context.verify_mode = ssl.CERT_NONE
            try:
                context.load_cert_chain(self.cert_filepath, keyfile=self.key_filepath)
            except OSError as exc:
                self._fail_with_exc_info("Cannot load SSL certificate, check files exist.", exc)
        def _fail_with_exc_info(self, arg0, exc):
            msg = arg0
            msg += "  More info: %s" % repr(exc)
            self.module.fail_json(msg=msg)
            # some ONTAP CLI commands return BEL on error
            # And 9.1 uses \r\n rather than \n !
            new_response = new_response.replace(b"\x07\r\n", b"")
            # And 9.7 may send backspaces
            for code_point in get_feature(self.module, "sanitize_code_points"):
                if bytes([8]) == b"\x08":   # python 3
                    byte = chr(code_point)
                new_response = new_response.replace(byte, b".")

        def _parse_response(self, response):
            try:
                return super()._parse_response(response)
            except zapi.etree.XMLSyntaxError as exc:
                if has_feature(self.module, "sanitize_xml"):
                    try:
                        return super()._parse_response(self.sanitize_xml(response))
                    except Exception:
                        pass
                with contextlib.suppress(Exception):
                    # report first exception, but include full response
                    exc.msg += ".  Received: %s" % response
