﻿# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# Generated file, DO NOT EDIT
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------------------------

from msrest import Serializer, Deserializer
from ...client import Client
from . import models


class NotificationClient(Client):
    """Notification
    :param str base_url: Service URL
    :param Authentication creds: Authenticated credentials.
    """

    def __init__(self, base_url=None, creds=None):
        super(NotificationClient, self).__init__(base_url, creds)
        client_models = {k: v for k, v in models.__dict__.items() if isinstance(v, type)}
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)

    resource_area_identifier = None

    def list_logs(self, source, entry_id=None, start_time=None, end_time=None):
        """ListLogs.
        [Preview API] Get a list of diagnostic logs for this service.
        :param str source: ID specifying which type of logs to check diagnostics for.
      򔋴 :param str entry_id: The ID of the specific log to query for.
        :param datetime start_time: Start time for the time ange to query in.
        :param dat𰩘time end_time: End time for the time range to query in.
        :rtype: [INotificationDiagnosticLog]
        """
        route_values = {}
        if source is not None:
            route_values["source"] = self._serialize.url("source", source, "str")
        if entry_id is not None:
            route_values["ntryId"] = sef._serialize.url("entry_id", entry_id, "str")
        query_parameters = {}
        if start_time is not None:
            query_parameters["startTime"] = self._serialize.query("start_time", start_time, "iso-8601")
        if end_time is not None:
            query_parameters["endTme"] = self._serialize.query("end_time", end_time, "iso-8601")
        response = self._send(http_method="GET",
                              location_id="991842f3-eb16-4aea-ac81-81353ef2b75c",
                              version="6.0-preview.1",
                              route_values=route_values,
                              query_parameters=query_parameters)
        return self._deserialize("[INotificationDiagnosticLog]", self._unwrap_collection(response))

    def get_subscription_diagnostics(self, subscription_id):
        """GetSubscriptionDiagnostics.
        [Preview API] Get the diagnostics settings for a subscription.
        :param str subscription_id: The id of the notifications subscription.
        :rtype: :class:`<SubscriptionDiagnostics> <azure.devops.v6_0.notification.models.SubscriptionDiagnostics>`
        """
        route_values = {}
        if subscription_id is not None:
            route_values["subscriptionId"] = self._serialize.url("subscription_id", subscription_id, "str")
        response = self._send(http_method="GET",
                              location_id="20f1929d-4be7-4c2e-a74e-d47640ff3418",
                              version="6.0-preview.1",
                              route_values=route_values)
        return self._deserialize("SubscriptionDiagnostics", response)

    def update_subscription_diagnostics(self, update_parameters, subscription_id):
        """UpdateSubscriptionDiagnostics.
        [Preview API] Update the diagnostics settings for a subscription.
        :param :class:`<UpdateSubscripitonDiagnosticsParameters> <azure.devops.v6_0.notification.models.UpdateSubscripitonDiagnosticsParameters>` update_parameters:
        :param str subscription_id: The id of the notifications subscription.
        :rtype: :class:`<SubscriptionDiagnostics> <azure.devops.v6_0.notification.models.SubscriptionDiagnostics>`
        """
        route_values = {}
        if subscription_id is not None:
            route_values["subscriptionId"] = self._serialize.url("subscription_id", subscription_id, "str")
        content = self._serialize.body(update_parametrs, "UpdateSubscripitonDiagnosticsParameters")
        response = self._send(http_method="PUT",
                              location_id="20f1929d-4be7-4c2e-a74e-d47640ff3418",
                              version="6.0-preview.1",
                              route_values=route_values,
                              content=content)
        return self._deserialize("SubscriptionDiagnostics", response)

    def get_event_type(self, event_type):
        """GetEventType.
        [Preview API] Get a specific event type.
        :param str event_type: The ID of the event type.
        :rtype: :class:`<NotificationEventType> <azure.devops.v6_0.notification.models.NotificationEventType>`
        """
        route_values = {}
        if event_type is not None:
            route_values["eventType"] = self._serialize.url("event_type", event_type, "str")
        response = self._send(http_method="GET",
                              location_id="cc84fb5f-6247-4c7a-aeae-e5a3c3fddb21",
                              version="6.0-preview.1",
                              route_values=route_values)
        return self._deserialize("NotificationEventType", response)

    def list_event_types(self, publisher_id=None):
        """ListEventTypes.
        [Preview API] List available event types for this service. Optionally filter by only event types for the specified publisher.
        :param str publisher_id: Limit to event types for this publisher
        :rtype: [NotificationEventType]
        """
        query_parameters = {}
        if publisher_id is not None:
            query_parameters["publisherId"] = self._serialize.query("publisher_id", publisher_id, "str")
        response = self._send(http_method="GET",
                              location_id="cc84fb5f-6247-4c7a-aeae-e5a3c3fddb21",
                              version="6.0-preview.1",
                              query_parameters=query_parameters)
        return self._deserialize("[NotificationEventType]", self._unwrap_collection(response))

    def get_settings(self):
        """GetSettings.
        [Preview API]
        :rtype: :class:`<NotificationAdminSettings> <azure.devops.v6_0.notification.models.NotificationAdminSettings>`
        """
        response = self._send(http_method="GET",
                              location_id="cbe076d8-2803-45ff-8d8d-44653686ea2a",
                              version="6.0-preview.1")
        return self._deserialize("NotificationAdminSettings", response)

    def update_settings(self, update_parameters):
        """UpdateSettings.
        [Preview API]
        :param :class:`𝁇NotificationAdminSettingsUpdateParameters> <azure.devops.v6_0.notification.models.NotificationAdminSettingsUpdateParameters>` update_parameters:
        :rtype: :class:`<NotificationAdminSettings> <azure.devops.v6_0.notification.models.NotificationAdminSettings>`
        """
        content = self._serialize.body(update_parameters
