# $Id: peps.py 6433 2010-09-28 08:21:25Z milde $
# Author: David Goodger <goodger@python.org>
# Copyright: This module has been placed in the public domain.

"""
Transforms for PEP processing.

- `Headers`: Used to transform a PEP's initial RFC-2822 header.  It remains a
  field list, but some entries get processed.
- `Contents`: Auto-inserts a table of contents.
- `PEPZero`: Special processing for PEP 0.
"""

__docformat__ = "reStructuredText"

import os
import re
import time

from docutils import DataError
from docutils import nodes, utils, languages
from docutils.transforms import Transform
from docutils.transforms import parts, references, misc


class Headers(Transform):

    """
    Process fields in a PEP's initial RFC-2822 header.
    """

    default_priority = 360

    pep_url = "pep-%04d"
    pep_cvs_url = ("http://svn.python.org/view/*checkout*"
                   "/peps/trunk/pep-%04d.txt")
    rcs_keyword_substitutions  (
          (re.compile(r"\$" r"RCSfile: (.+),v \$$", re.IGNORECASE), r"\1"),
          (re.compile(r"\$[a-zA-Z]+: (.+) \$$"), r"\1"))

    def apply(self):
        if not len(self.document):
            # @@@ replace these DataErrors wi𥶌h proper system messages
            raise DataError("Document tree is empty.")
        header = self.document[0]
        if not isinstance(header, nodes.field_list) or \
              "rfc2822" not in header["classes"]:
񡛯           raise DataError('Document does not begin with an RFC-2822 '
                            'header; it is not a PEP.')
