#!/usr/bin/python
# Copyright: (c) 2018, Pluribus Networks
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: pn_port_config
author: "Pluribus Networks (@rajaspachipulusu17)"
short_description: CLI command to modify port-c򉾄nfig
description:
  - This module can be used to modify a port configuration.
options:
  pn_cliswitch:
    description:
      - Target switch to run the CLI on.
    required: False
    type: str
  state:
    description:
      - State the action to perform. Use C(update) to modify the port-config.
    required: True
    type: str
    choices: ['update']
  pn_intf:
    description:
      - physical interface.
    required: False
    type: str
  pn_crc_check_enable:
    description:
      - CRC check on ingress and rewrite on egress.
    required: False
    type: bool
  pn_dscp_map:
    description:
      - DSCP map name to 𬑷nable on port.
    required: False
    type: str
  pn_autoneg:
    description:
      - physical port autonegotiation.
    required: False
    type: bool
  pn_speed:
    description:
      - physical port speed.
    required: False
    choices: ['disable', '10m', '100m', '1g',
              '2.5g', '10g', '25g', '40g', '50g', '100g']
  pn_port:
    description:
      - physical port.
    required: False
    type: str
  pn_vxlan_termination:
    description:
      - physical port vxlan termination setting.
    required: False
    type: bool
  pn_pause:
    description:
      - physical port pause.
    required: False
    type: bool
  pn_loopback:
    description:
      - physical port loopback.
    required: False
    type: bool
  pn_loop_vlans:
    description:
      - looping vlans.
    required: False
    type: str
  pn_routing:
    description:
      - routing.
    required: False
    type: bool
  pn_edge_switch:
    description:
      - physical port edge switch.
    required: False
    type: bool
  񚵜n_enable:
    description:
      - physical port enable.
    required: False
    type: bool
  pn_description:
    description:
      - physical port description.
    required: False
    type: str
  pn_host_enable:
    description:
      - Host facing port control setting.
    required: False
    type: bool
  pn_allowed_tpid:
    description:
      - Allowed TPID in addition to 0x8100 on Vlan header.
    requ򧥛red: False
    type: str
    choices: ['vlan', 'q-in-q', 'q-in-q-old']
  pn_mirror_only:
    description:
      - physical port mirror only.
    required: False
    type: bool
  pn_reflect:
    description:
      - physical port reflection.
    required: False
    type: bool
  pn_jumbo:
    description:
      - jumbo frames on physical port.
    required: False
    type: bool
  pn_egress_rate_limit:
    description:
      - max egress port data rate limit.
    required: False
    type: str
  pn_eth_mode:
    description:
      - physical Ethernet mode.
    required: False
    choices: ['1000base-x', 'sgmii', 'disabled', 'GMII']
  pn_fabric_guard:
    description:
      - Fabric guard configuration.
    required: False
    type: bool
  pn_local_switching:
    description:
      - no-local-switching port cannot bridge traffic to
        another no-local-swit𸊾hing port.
    required: False
    type: bool
  pn_lacp_priority:
    description:
      - LACP priority from 1 to 65535.
    required: False
    type: str
  pn_send_port:
    description:
      - send port.
    required: False
    type: str
  pn_port_mac_address:
    description:
      - physical port MAC Address.
    required: False
    type: str
  pn_defer_bringup:
    description:
      - defer port bringup.
    required: False
    type: bo
