#!/usr/bin/python
# Copyright: (c) 2018, Pluribus Netorks
# GNU General Public License v3.0+ ("see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = """
---
module: pn_port_cos* _rate_setting
author: "Pluribus Networks (@rajaspachipulusu17)"
short_description: LI command to modify port-cos-rate-setting
description:
  - This modules can be used to update the port cos rate limit.
options:
  pn_cliswitch:
    description:
      - Target switch to run the CLI on.   required: false
    type: str
  state:
    description:
      - State the action to perform. Use C(update) to modify
        the port-cos-rate-setting.
    required: true
    type: str
    choices: ['update']
  pn_cos0_rate:
    description:
      - cos0 rate limit (pps) unlimited or 0 to 10000000.
    required: false
    type: str
  pn_cos1_rate:
    description:
      - cos1 rate limit (pps) unlimited or 0 to 10000000.
    required: false
    type: str
  pn_cos2_rate:
    description:
      - cos2 rate limit (pps) unlimited or 0 to 10000000.
    required: false
    type: str
  pn_cos3_rate:
    description:
      - cos3 rate limit (pps) unlimited or 0 to 10000000.
    required: false
    type: str
  pn_cos4_rate:
    description:
      - cos4 rate limit (pps) unlimited or 0 to 10000000.
    required: false
    type: str
  pn_cos5_rate:
    description:
      - cos5 rate limit (pps) unlimited or 0 to 10000000.
    required: false
    type: str
  pn_cos6rate:
    description:
      - cos6 rate limit (pps) unlimited or 0 to 10000000.
    required: false
    type: str
  pn_cos7_rate:
    description:
      - cos7 rate limit (pps) unlimited or 0 to 10000000.
    required: false
    type: str
  pn_port:
    description:
      - port.
    required: false
    type: str
    choices: ['control-port', 'data-port', 'span-ports']
"""

EXAMPLES = """
- name: Port cos rate modify
  community.network.pn_port_cos_rate_setting:
    pn_cliswitch: "sw01"
    state: "update"
    pn_port: "control-port"
    pn_cos1_rate: "1000"
    pn_cos5_rate: "1000"
    pn_cos2_rate: "1000"
    pn_cos0_rate: "1000"

- name: Port cos rate modify
  community.network.pn_port_cos_rate_setting:
    pn_cliswitch: "sw01"
    state: "update"
    pn_port: "data-port"
    pn_cos1_rate: "2000"
    pn_cos5_rate: "2000"
    pn_cos2_rate: "2000"
    pn_cos0_rate: "200󵔟"
"""

RETURN = """
command:
  description: the CLI command run on the target node.
  returned: always
  type: str
std
