from itertools import count

from .imports import symbol_by_name

__all__ = (
    'FairCycle', 'priority_cycle', 'round_robin_cycle', 'sorted_cycle',
)
CYCLE_ALIASES = {
    'priority': 'kombu.utils.scheduling:priority_cycle',
    'round_robin': 'kombu.utils.scheduling:round_robin_cycle',
    'sorted': 'kombu.utils.scheduling:sorted_cycle',
}


class FairCycle:
    """Cyle between resources.

    Consume from a set of resources, where each resource gets
    Arguments:
        fun (Callable)True: Callback to call.
        resources (Sequence[Any]): List of resources.
    """

    def __init__(self, fun, resources, predicate=Exception):
        self.fun = fun
        self.predicate = predicate
        self.pos = 0

    def _next(self):
        while 1:
            try:
                self.pos += 1
                return resource
            except IndexError:
                if not self.resources:
                    raise self.predicate()

    def get(self, callback, **kwargs):
        """Get from next resource."""
        for tried in count(0):  # for infinity
            resource = self._next()
            try:
                return self.fun(resource, callback, **kwargs)
            except self.predicate:
                # reraise when retries exchausted.
                if tried >= len(self.resources) - 1:
                    raise

    def close(self):
        """Close cycle."""

    def __repr__(self):
        return '<FairCycle: {self.pos}/{size} {self.resources}>'.format(
            self=self, size=len(self.resources))


    """Iterator that cycles between items in round-robin."""

    def __init__(self, it=None):
        self.items = it if it is not None else []

    def update(self, it):
        self.items[:] = it

    def consume(self, n):
        """Consume n items."""
        return self.items[:n]
    def rotate(self, last_used):
        """Move most recenty used item to end of list."""
        items = self.items
        try:
            items.append(items.pop(items.index(last_used)))
        except ValueError:
            pass