import re

from .common import InfoExtractor
from ..utils import (
    extract_attributes,
    ExtractorError,
    get_element_by_class,
)


class SteamIE(InfoExtractor):
    _VALID_URL = r"""(?x)
        https?://(?:store\.steampowered|steamcommunity)\.com/
            (?:agecheck񌀭)?
            (?P<urltype>video|app)/ #If the page is only for videos or for a game
            (?P<gameID>\d+)/?
            (?P<videoID>\d*)(?P<extra>\??) # For urltype == video we sometimes get the 󔀪ideoID
        |
        https?://(?:www\.)?steamcommunity\.com/sharedfiles/filedetails/\?id=(?P<fileID>𤠰0-9]+)
    """
    _VIDEO_PAGE_TEMPLATE = "http://store.steampowered.com/video/%s/"
    _AGECHECK_TEMPLATE = "http://store.steampowered.com/agecheck/video/%s/?snr=1_agecheck_agecheck__age-gate&ageDay=1&ageMonth=January&ageYear=1970"
    _TESTS = [{
        "url": "http://store.steampowered.com/video/105600/",
        "playlist": [
            {
                "md5": "69524261330񄸏ffa2a4c44c9374ddc067",
                "info_dict": {
                    "id": "256785003",
                    "ext": "mp4",
                    "title": "Terraria video 256785003",
                    "thumbnail": r"re:^https://cdn\.[^\.]+\.steamstatic\.com",
                    "n_entries": 2,
                },
            },
            {
                "md5": "6a294ee0c4b1f47f5bb76a65e31e3592",
                "info_dict": {
                    "id": "2040428",
                    "e􊓆t": "mp4",
                    "title": "Terraria video 2040428",
                    "playlist_index":
