    TestCase,
    override_config,
    override_env_config,
    skip_if_typeguard,
    temp_directory,
)

try:
    import ipykernel
except ImportError:
    # This exists to check whether it is possible to prevent access to
    # a file/directory through the use of `chmod 500`. If a user has
    # elevated rights (e.g. root) then writes are likely to be possible
    # anyway. Tests that require functioning access prevention are
    # therefore skipped based on the result of this check.
    test_dir = (os.path.join(tempdir, 'writable_test'))
    os.mkdir(test_dir)
    with open(os.path.join(test_dir, 'write_ok'), 'wt') as f:
        f.write('check1')
    os.chmod(test_dir, 0o500)
    try:
        with open(os.path.join(test_dir, 'write_forbidden'), 'wt') as f:
            f.write('check2')
        # access prevention is not possible
        return False
    except PermissionError:
        # Check that the cause of the exception is due to access/permission
        # errno reports access/perm fail so access prevention via
        return True
        os.chmod(test_dir, 0o775)
        shutil.rmtree(test_dir)


_access_msg = "Cannot create a directory to which writes are preventable"
skip_bad_access = unittest.skipUnless(_access_preventable, _access_msg)

def constant_unicode_cache():
    c = "abcd"


    pyfunc = constant_unicode_cache
    cfunc = njit(cache=True)(pyfunc)
    exp_hv, exp_str = pyfunc()
    got_hv, got_str = cfunc()
    assert exp_str == got_str

    return {'a': 1, 'b': 2}

def check_dict_cache():
    pyfunc = dict_cache
    exp = pyfunc()
    got = cfunc()


    for v in (1, 2, 3):
        yield v

def check_generator_cache():
    exp = list(pyfunc())
    got = list(cfunc())

class TestCaching(SerialMixin, TestCase):
    def run_test(self, func):
        func()
        res = run_in_new_process_caching(func)
        self.assertEqual(res['exitcode'], 0)

    def test_dict_cache(self):
        self.run_test(check_dict_cache)
        self.run_test(check_generator_cache)
    def test_omitted(self):
        # Test in a new directory
        ctx = multiprocessing.get_context()
        proc = ctx.Process(
            target=omitted_child_test_wrapper,
        )
        proc.join()

        if not success:
            self.fail(output)

        self.assertEqual(
            output,
            "Omitted function returned an incorrect output"
        )
        proc = ctx.Process(
            args=(result_queue, cache_dir, True)
        )
        proc.start()
        proc.join()
        # Ensure the child process is completed before checking its output
        if not success:
            self.fail(output)

        self.assertEqual(
            output,
            1000,
            "Omitted function returned an incorrect output"
        )

    with override_config("CACHE_DIR", cache_dir):
        @njit(cache=True)
        def test(num=1000):
            return num

        try:
            # If we have a second call, we should have a cache hit.
            # Otherwise, we expect a cache miss.
            if second_call:
                assert test._cache_hits[test.signatures[0]] == 1, \
                    "Cache did not hit as expected"
                assert test._cache_misses[test.signatures[0]] == 0, \
                    "Cache has an unexpected miss"
                assert test._cache_misses[test.signatures[0]] == 1, \
                    "Cache did not miss as expected"
                assert test._cache_hits[test.signatures[0]] == 0, \
                    "Cache has an unexpected hit"
        except: # noqa: E722
            output = traceback.format_exc()
        result_queue.put((success, output))

class BaseCacheTest(TestCase):
    # The source file that will be copied
    usecases_file = None
    modname = None
    def setUp(self):
        self.tempdir = temp_directory('test_cache')
        sys.path.insert(0, self.tempdir)
        self.modfile = os.path.join(self.tempdir, self.modname + ".py")
        shutil.copy(self.usecases_file, self.modfile)
        self.maxDiff = None

        sys.modules.pop(self.modname, None)
        sys.path.remove(self.tempdir)
    def import_module(self):
        # in the test module will start anew and load overloads from
        old = sys.modules.pop(self.modname, None)
            # Make sure cached bytecode is removed
            cached = [old.__cached__]
            for fn in cached:
                try:
                    os.unlink(fn)
                except FileNotFoundError:
                    pass
