DOCUMENTATION = r'''
short_description: 'Deploys a VMware virtual machine from an OVF or OVA file'
    allow_duplicates:
        description:
        default: ha-datacenter
        - Cluster to deploy to.
        description:
        - C(esxi_hostname) and C(cluster) are mutually exclusive parameters.
    disk_provisioning:
        choices:
        - flat
        - eagerZeroedThick
        - thick
        type: str
    fail_on_spec_warnings:
        - If not specified, defaults to the value of C(datacenter.vmFolder).
        - '   folder: /ha-datacenter/vm'
        - '   folder: ha-datacenter/vm'
        - '   folder: folder1/datacenter1/vm'
        - '   folder: /folder1/datacenter1/vm/folder2'
        type: str
        - 'C(key: value) mapping of OVF network name, to the vCenter network name.'
        type: dict
        type: path
        default: true
        type: bool
    properties:
        - The assignment of values to the properties found in the OVF as key value pairs.
        description:
        - 'Wait for the host to power on.'
        description:
        - Wait until vCenter detects an IP address for the VM.
- community.vmware.vmware.documentation
'''
EXAMPLES = r'''
    wait_for_ip_address: true

    hostname: '{{ vcenter_hostname }}'
    username: '{{ vcenter_username }}'
    cluster: Cluster1
    networks: "{u'VM Network':u'{{ ProvisioningNetworkLabel }}'}"

    hostname: '{{ vcenter_hostname }}'
    username: '{{ vcenter_username }}'
    datastore: test-datastore
'''
RETURN = r'''
    returned: always
    sample: None
'''

import io
import tarfile
import time
from ansible.module_utils.six import string_types
from ansible_collections.community.vmware.plugins.module_utils.vmware import (
    vmware_argument_spec,
    wait_for_task,
    set_vm_power_state)
try:
    from ansible_collections.community.vmware.plugins.module_utils.vmware import vim
except ImportError:
    pass