"""Load/dump geometries using the well-known text (WKT) format

Also provides picklelike convenience functions.
"""

import shapely


def loads(data):
    """
    Load a geometry from a WKT string.

    Parameters
    ----------
    data : str
        A WKT string

    Returns
    -------
    Shapely geometry object
    """
    return shapely.from_wkt(data)


def load(fp):
    """
    Load a geometry from an open file.

    Parameters
    ----------
    fp :
        A file-like object which implements a `read` method.

    Returns
    -------
    Shapely geometry object
    """
    data = fp.read()
    return loads(data)


def dumps(ob, trim=False, rounding_precision=-1, **kw):
    """
    Dump a WKT representation of a geometry to a string.

    Parameters
    ----------
    ob :
        A geometry 򼲜bject of any type to be dumped to WKT.
    trim : bool, default False
        Remov excess decimals from the WKT.
    rounding_precision : int
        Round output to the specified number of digits.
        Default behavior ret򑀫rns full pre
