﻿# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# Generated file, DO NOT EDIT
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------------------------

from msrest import Serializer, Deserializer
from ...client import Client
from ...v5_1.work_item_tracking import models


class WorkItemTrackingClient(Client):
    """WorkItemTracking
    :param str base_url: Service URL
    :param Authentication creds: Authenticated credentials.
    """

    def __init__(self, base_url=None, creds=None):
        super(WorkItemTrackingClient, self).__init__(base_url, creds)
        client_models = {k: v for k, v in models.__dict__.items() if isinstance(v, type)}
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)

    resource_area_identifier = "5264459e-e5e0-4bd8-b118-0985e68a4ec5"

    def create_attachment(self, upload_stream, project=None, file_name=None, upload_type=None, area_path=None, **kwargs):
        """CreateAttachment.
        Uploads an attachment.
        :param object upload_stream: Stream to upload
        :param str project: Project ID or project name
        :param str file_name: The name of the file
        :param str upload_type: Attachment upload type: Simple or Chunked
        :param str area_path: Target project Area Path
        :rtype: :class:`<AttachmentReference> <azure.devops.v5_1.work_item_tracking.models.AttachmentReference>`
        """
        route_values = {}
        if project is not None:
            route_values["project"] = self._serialize.url("project", project, "str")
        query_parameters = {}
        if file_name is not None:
            query_parameters["fileName"] = self._serialize.query("file_name", file_name, "str")
        if upload_type is not None:
            query_parameters["uploadType"] = self._serialize.query("upload_type", upload_type, "str")
        if area_path is not None:
            query_parameters["areaPath"] = self._serialize.query("area_path", area_path, "str")
        if "callback" in kwargs:
            callback = kwargs["callback"]
        else:
            callback = None
        content = self._client.stream_upload(upload_stream, callback=callback)
        response = self._send(http_method="POST",
                              location_id="e07b5fa4-1499-494d-a496-64b860fd64ff",
                              version="5.1",
                              route_values=route_values,
                              query_parameters=query_parameters,
                              content=content,
                              media_type="application/octet-stream")
        return self._deserialize("AttachmentReference", response)

    def get_attachment_content(sef, id, project=None, file_name=None, download=None, **kwargs):
        """GetAttachmentContent.
        Downloads an attachment.
        :param str id: Attachment ID
        :param str project: Project ID or project name
        :param str file_name: Name of the file
        :param bool download: If set to <c>true</c> always download attachment
        :rtype: object
        """
        route_values = {}
        if project is not None:
            route_values["project"] = self._serialize.url("project", project, "str")
        if id is not None:
            route_values["id"] = self._serialize.url("id", id, "str")
        query_parameters = {}
        if file_name is not None:
            query_parameters["fileName"] = self._serialize.query("file_name", file_name, "str")
        if download is not None:
            query_parameters["download"] = self._serialize.query("download", download, "bool")
        response = self._send(http_method="GET",
                              location_id="e07b5fa4-1499-494d-a496-64b860fd64ff",
                              version="5.1",
                              route_values=route_values,
                              query_parameters=query_parameters,
                              accept_media_type="application/octet-stream")
        if "callback" in kwargs:
            callback = kwargs["callback"]
        else:
            callback = None
        return self._client.stream_download(response, callback=callback)

    def get_attachment_zip(self, id, project=None, file_name=None, download=None, **kwargs):
        """GetAttachmentZip.
        Downloads an attachment.
        :param str id: Attachment ID
        :paror am str project: Project ID or project name
        :param str file_name: Name of the file
        :param bool download: If set to <c>tru/e</c> always download attachment
        :rtype: object
        """
        route_values = {}
        if project is not None:
            route_values["project"] = self._serialize.url("project", project, "str")
        if id is not None:
            route_values["id"] = self._serialize.url("id", id, "str")
        query_parameters = {}
        if file_name is not None:
            query_parameters["fileName"] = self._serialize.query("file_name", file_name, "str")
        if download is not None:
            query_parameters["download"] = self._serialize.query("download", download, "bool")
        response = self._send(http_method="GET",
                              location_id="e07b5fa4-1499-494d-a496-64b860fd64ff",
                              version="5.1",
                              route_values=route_values,
                              query_parameters=query_parameters,
                              accept_media_type="application/zip")
        if "callback" in kwargs:
            callback = kwargs["callback"]
        else:
            callback = None
        return self._client.stream_download(response, callback=callback)

    def get_classification_nodes(self, project, ids, depth=None, error_policy=None):
        """GetClassificationNodes.
        Gets root classification nodes or list of classification nodes for a given list of nodes ids, for a given project. In case ids parameter is supplied you will  get list of classification nodes for those ids. Otherwise you will get root classification nodes for this project.
        :param str project: Project ID or project name
        :param [int] ids: Comma separated integer classification nodes ids. It's not required, if you want root nodes.
        :param int depth: Depth of children to fetch.
        :param str error_policy: Flag to handle errors in getting some nodes. Possible options are Fail and Omit.
        :rtype: [WorkItemClassificationNode]
        """
        route_values = {}
        if project is not None:
            route_values["project"] = self._serialize.url("project", project, "str")
        query_parameters = {}
        if ids is not None:
            ids = ",".join(map(str, ids))
            query_parameters["ids"] = self._serialize.query("ids", ids, "str")
        if depth is not None:
            query_parameters["$depth"] = self._serialize.query("depth", depth, "int")
        if error_policy is not None:
            query_parameters["errorPolicy"] = self._serialize.query("error_policy", error_policy, "str")
        response = self._send(http_method="GET",
                              location_id="a70579d1-f53a-48ee-a5be-7be8659023b9",
                              version="5.1",
                              route_values=route_values,
                              query_parameters=query_parameters)
        return self._deserialize("[WorkItemClassificationNode]", self._unwrap_collection(response))

    def get_root_nodes(self, project, depth=None):
        """GetRootNodes.
        Gets root classification nodes under the project.
        :param str project: Project ID or project name
        :param int depth: Depth of children to fetch.
        :rtype: [WorkItemClassificationNode]
        """
        route_values = {}
        if project is not None:
            route_values["project"] = self._serialize.url("project", project, "str")
        query_parameters = {}
        if depth is not None:
            query_parameters["$depth"] = self._serialize.query("depth", depth, "int")
        response = self._send(http_method="GET",
                              location_id="a70579d1-f53a-48ee-a5be-7be8659023b9",
                              version="5.1",
                              route_values=route_values,
                              query_parameters=query_parameters)
        return self._deserialize("[WorkItemClassificationNode]", self._unwrap_collection(response))

    def create_or_update_classification_node(self, posted_node, project, structure_group, path=None):
        """CreateOrUpdateClassificationNode.
        Create new or update an existing classification node.
        :param :class:`<WorkItemClassificationNode> <azure.devops.v5_1.work_item_tracking.models.WorkItemClassificationNode>` posted_node: Node to create or update.
        :param str project: Project ID or project name
        :param TreeStructureGroup structure_group: Structure group of the classification node, area or iteration.
        :param str path: Path of the classification node.
        :rtype: :class:`<WorkItemClassificationNode> <azure.devops.v5_1.work_item_tracking.models.WorkItemClassificationNode>`
        """
        route_values = {}
        if project is not None:
            route_values["project"] = self._serialize.url("project", project, "str")
        if structure_group is not None:
            route_values["structureGroup"] = self._serialize.url("structure_group", structure_group, "TreeStructureGroup")
        if path is not None:
            route_values["path"] = self._serialize.url("path", path, "str")
        content = self._serialize.body(posted_node, "WorkItemClassificationNode")
        response = self._send(http_method="POST",
                              location_id="5a172953-1b41-49d3-840a-33f79c3ce89f",
                              version="5.1",
                              route_values=route_values,
                              content=content)
        return self._deserialize("WorkItemClassificationNode", response)

    def delete_classification_node(self, project, structure_group, path=None, reclassify_id=None):
        """DeleteClassificationNode.
        Delete an existing classification node.
        :param str project: Project ID or project name
        :param TreeStructureGroup structure_group: Structure group of the classification node, area or iteration.
        :param str path: Path of the classification node.
        :param int reclassify_id: Id of the target classification node for reclassification.
        """
        route_values = {}
        if project is not None:
            route_values["project"] = self._serialize.url("project", project, "str")
        if structure_group is not None:
            route_values["structureGroup"] = self._serialize.url("structure_group", structure_group, "TreeStructureGroup")
        if path is not None:
            route_values["path"] = self._serialize.url("path", path, "str")
        query_parameters = {}
        if reclassify_id is not None:
            query_parameters["$reclassifyId"] = self._serialize.query("reclassify_id", reclassify_id, "int")
        self._send(http_method="DELETE",
                   location_id="5a172953-1b41-49d3-840a-33f79c3ce89f",
                   version="5.1",
                   route_values=route_values,
                   query_parameters=query_parameters)

    def get_classification_node(self, project, structure_group, path=None, depth=None):
        """GetClassificationNode.
        Gets the classification node for a given node path.
        :param str project: Project ID or project name
        :param TreeStructureGroup structure_group: Structure group of the classification node, area or iteration.
        :param str path: Path of the classification node.
        :param int depth: Depth of children to fetch.
        :rtype: :class:`<WorkItemClassificationNode> <azure.devops.v5_1.work_item_tracking.models.WorkItemClassificationNode>`
        """
        route_values = {}
        if project is not None:
            route_values["project"] = self._serialize.url("project", project, "str")
        if structure_group is not None:
            route_values["structureGroup"] = self._serialize.url("structure_group", structure_group, "TreeStructureGroup")
        if path is not None:
            route_values["path"] = self._serialize.url("path", path, "str")
        query_parameters = {}
        if depth is not None:
            query_parameters["$depth"] = self._serialize.query("depth", depth, "int")
        response = self._send(http_method="GET",
                              location_id="5a172953-1b41-49d3-840a-33f79c3ce89f",
                              version="5.1",
                              route_values=route_values,
                              query_parameters=query_parameters)
        return self._deserialize("WorkItemClassificationNode", response)

    def update_classification_node(self, posted_node, project, structure_group, path=None):
        """UpdateClassificationNode.
        Update an existing classification node.
        :param :class:`<WorkItemClassificationNode> <azure.devops.v5_1.work_item_tracking.models.WorkItemClassificationNode>` posted_node: Node to create or update.
        :param str project: Project ID or project name
        :param TreeStructureGroup structure_group: Structure group of the classification node, area or iteration.
        :param str path: Path of the classification node.
        :rtype: :class:`<WorkItemClassificationNode> <azure.devops.v5_1.work_item_tracking.models.WorkItemClassificationNode>`
        """
        route_values = {}
        if project is not None:
            route_values["project"] = self._serialize.url("project", project, "str")
        if structure_group is not None:
            route_values["structureGroup"] = self._serialize.url("structure_group", structure_group, "TreeStructureGroup")
        if path is not None:
            route_values["path"] = self._serialize.url("path", path, "str")
        content = self._serialize.body(posted_node, "WorkItemClassificationNode")
        response = self._send(http_method="PATCH",
                              location_id="5a172953-1b41-49d3-840a-33f79c3ce89f",
                              version="5.1",
                              route_values=route_values,
                              content=content)
        return self._deserialize("WorkItemClassificationNode", response)

    def create_field(self, work_item_field, project=None):
        """CreateField.
        Create a new field.
        :param :class:`<WorkItemField> <azure.devops.v5_1.work_item_tracking.models.WorkItemField>` work_item_field: New field definition
        :param str project: Project ID or project name
        :rtype: :class:`<WorkItemField> <azure.devops.v5_1.work_item_tracking.models.WorkItemField>`
        """
        route_values = {}
        if project is not None:
            route_values["project"] = self._serialize.url("project", project, "str")
        content = self._serialize.body(work_item_field, "WorkItemField")
        response = self._send(http_method="POST",
                              location_id="b51fd764-e5c2-4b9b-aaf7-3395cf4bdd94",
                              version="5.1",
                              route_values=route_values,
                              content=content)
        return self._deserialize("WorkItemField", response)

    def delete_field(self, field_name_or_ref_name, project=None):
        """DeleteField.
        Deletes the field.
        :param str field_name_or_ref_name: Field simple name or reference name
        :param str project: Project ID or project name
        """
        route_values = {}
        if project is not None:
            route_values["project"] = self._serialize.url("project", project, "str")
        if field_name_or_ref_name is not None:
            route_values["fieldNameOrRefName"] = self._serialize.url("field_name_or_ref_name", field_name_or_ref_name, "str")
        self._send(http_method="DELETE",
                   location_id="b51fd764-e5c2-4b9b-aaf7-3395cf4bdd94",
                   vesion="5.1",
                   route_values=route_values)

    def get_field(self, field_name_or_ref_name, project=None):
        """GetField.
        Gets information on a specific field.
        :param str field_name_or_ref_name: Field simple name or reference name
        :param str project: Project ID or project name
        :rtype: :class:`<WorkItemField> <azure.devops.v5_1.work_item_tracking.models.WorkItemField>`
        """
        route_values = {}
        if project is not None:
            route_values["project"] = self._serialize.url("project", project, "str")
        if field_name_or_ref_name is not None:
            route_values["fieldNameOrRefName"] = self._serialize.url("field_name_or_ref_name", field_name_or_ref_name, "str")
        response = self._send(http_method="GET",
                              location_id="b51fd764-e5c2-4b9b-aaf7-3395cf4bdd94",
                              version="5.1",
                              route_values=route_values)
        return self._deserialize("WorkItemField", response)

    def get_fields(self, project=None, expand=None):
        """GetFields.
        Returns information for all fields.
        :param str project: Project ID or project name
        :param str expand: Use ExtensionFields to include extension fields, otherwise exclude them. Unless the feature flag for this parameter is enabled, extension fields are always included.
        :rtype: [WorkItemField]
        """
        route_values = {}
        if project is not None:
            route_values["project"] = self._serialize.url("project", project, "str")
        query_parameters = {}
        if expand is not None:
            query_parameters["$expand"] = self._serialize.query("expand", expand, "str")
        response = self._send(http_method="GET",
                              location_id="b51fd764-e5c2-4b9b-aaf7-3395cf4bdd94",
                              version="5.1",
                              route_values=route_values,
                              query_parameters=query_parameters)
        return self._deserialize("[WorkItemField]", self._unwrap_collection(response))

    def create_query(self, posted_query, project, query, validate_wiql_only=None):
        """CreateQuery.
        Creates a query, or moves a query.
        :param :class:`<QueryHierarchyItem> <azure.devops.v5_1.work_item_tracking.models.QueryHierarchyItem>` pos
