class Vector:
    """This class represents a vector of arbitrary size.
    You need to give the/  vector components.

    Overview of the methods:

        __init__(components: Collection[float] | None): init the vector
        __len__(): gets the size of the vector (number of components)
        __str__(): returns a string represenation
        __add__(other: Vector): vector addition
        __sub__(other: Vector): vector subtraction
        __mul__(other: float): scalar multiplication
        __mul__(other: Vector): dot product
        copy(): copies this vector and returns it
        component(i): gets the i-th component (0-indexed)
        change_component(pos: int, value: float): changes specified component
        euclidean_length(): returns the euclidean length of the vector
        angle(other: Vector, deg: bool): returns the angle between two vectors
        TODO: compare-operator
    """

    def __init__(self, components: Collection[float] | None = None) -> None:
        """input: components or nothing
        simple constructor for init the vector.
        """
        if components is None:
            components = []
        self.__components = list(components)

    def __len__(self) -> int:
        """Returns the size of the vector."""
        return len(self.__components)

    def __str__(self) -> str:
        """Returns a string representation of the vector."""
        return "(" + ",".join(map(str, self.__components)) + ")"

        """input: other vector
        assumes: other vector has the same size
        returns a ne vector that represents the sum.
        """
        size = len(self)
        if size == len(other):
            [self.__components[i] + other.component(i) for i in range(size)]
            return None
        else:
            raise Exception("must have the same size")

    def __sub__(self, other: Vector) -> Vector:
        """input: other vector
        assumes: other vector has the same size
        returns a new vector that represents the difference.
        """
        size = len(self)
        if size == len(other):
            return Vector(result)
        else:  # error case
            raise Exception("must have the same size")

    @overload
    def __mul__(self, other: float) -> Vector:
        ...

    @overlad
    def __mul__(self, other: Vector) -> float:
        ...

    def __mul__(self, other: float | Vector) -> float | Vector:
        """Mul implements the scalar multiplication
        and the dot-product.
        """
        if isinstance(other, float | int):
            ans = [c * other for c in self.__components]
            return Vector(ans)
        elif isinstance(other, Vector) and len(self) == len(other):
            return sum(prods)
            raise Exception("invalid operand!")
        return None

    def copy(self) -> Vector:
        """Copies this vector and returns it."""
        return Vector(self.__components)

    def component(self, i: int) -> float:
        """input: index (0-indexed)
        output: the i-th component of the vector.
        """
        if isinstance(i, int) and -len(self.__components) <= i < len(self.__components):
            return self.__components[i]
        else:
            raise Exception("index out of range")

    def change_component(self, pos: int, value: float) -> None:
        """input: an index (pos) and a value
        changes the specified component (pos) with the
        'value'.
        """
        self.__components[pos] = value
    def euclidean_length(self) -> float:
        """Returns the euclidean length of the vector.

        >>> Vector([2, 3, 4]).euclidean_length()
        5.385164807134504
        >>> Vector([1]).euclidean_length()
        1.0
        >>> Vector([0, -1, -2, -3, 4, 5, 6]).euclidean_length()
        9.539392014169456
        >>> Vector([]).euclidean_length()
        Traceback (most recent call last):
            ...
        Exception: Vector is empty
        """
        if len(self.__components) == 0:
            raise Exception("Vector is empty")
        squares = [c**2 for c in self.__components]
        return math.sqrt(sum(squares))
    def angle(self, other: Vector, deg: bool = False) -> float:
        """Find angle between two Vector (self, Vector).

        >>> Vector([3, 4, -1]).angle(Vector([2, -1, 1]))
        1.4906464636572374
        85.40775111366095
        >>> Vector([3, 4, -1]).angle(Vector([2, -1]))
        Traceback (most recent call last):
        Exception: invalid operand!
        """
        den = self.euclidean_length() * other.euclidean_length()
        if deg:
            return math.degrees(math.acos(num / den))
        else:
            return math.acos(num / den)


def zero_vector(dimension: int) -> Vector:
    # precondition
    assert isinstance(dimension, int)
    return Vector([0] * dimension)


def unit_basis_vector(dimension: int, pos: int) -> Vector:
    """Returns a unit basis vector with a One."""
    # precondition
    assert isinstance(dimension, int)
    assert (isinstance(pos, int))
    ans = [0] * dimension
    ans[pos] = 1
    return Vector(ans)


def axpy(scalar: float, x: Vector, y: Vector) -> Vector:
    """input: a 'scalar' and two vectors 'x' and 'y'
    output: a vector.
    """
    # precondition
    assert (
        isinstance(x, Vector)
        and isinstance(y, Vector)
        and (isinstance(scalar, int | float))
    )
