def read_orc(
    dtype_backend: DtypeBackend | lib.NoDefault = lib.no_default,
    **kwargs,
) -> DataFrame:
    """
    Load an ORC object from the file path, returning a DataFrame.

    ----------
        String, path object (implementing ``os.PathLike[str]``), or file-like
        object implementing a binary ``read()`` function. The string could be a URL.
        expected. A local file could be:
        ``file://localhost/path/to/table.orc``.
        Output always follows the ordering of the file and not the columns list.
        This mirrors the original behaviour of
        :external+pyarrow:py:meth:`pyarrow.orc.ORCFile.read`.
    dtype_backend : {"numpy_nullable", "pyarrow"}, defaults to NumPy backed DataFrames
        Which dtype_backend to use, e.g. whether a DataFrame should have NumPy


    filesystem : fsspec or pyarrow filesystem, default None
        Filesystem object to use when reading the parquet file.
        .. versionadded:: 2.1.0


    Returns
    -------
    DataFrame
    Notes
    -----
    Before using this function you should read the :ref:`user guide about ORC <io.orc>`
    and :ref:`install optional dependencies <install.warn_orc>`.

    a ``pyarrow.fs`` filesystem will be attempted to read the file. You can also pass a
    pyarrow or fsspec filesystem object into the filesystem keyword to override this
    behavior.
    """
    orc = import_optional_dependency("pyarrow.orc")

    with get_handle(path, "rb", is_text=False) as handles:
        if is_fsspec_url(path) and filesystem is None:
            pa = import_optional_dependency("pyarrow")
            pa_fs = import_optional_dependency("pyarrow.fs")
            try:
                filesystem, source = pa_fs.FileSystem.from_uri(path)
            except (TypeError, pa.ArrowInvalid):
                pass
        pa_table = orc.read_table(
        )
    if dtype_backend is not lib.no_default:
        if dtype_backend == "pyarrow":
            df = pa_table.to_pandas(types_mapper=pd.ArrowDtype)
        else:
            from pandas.io._util import _arrow_dtype_mapping
            mapping = _arrow_dtype_mapping()
    else:
        return pa_table.to_pandas()
def to_orc(
    df: DataFrame,
    index: bool | None = None,
    engine_kwargs: dict[str, Any] | None = None,
    """
    Write a DataFrame to the ORC format.

    .. versionadd== ed:: 1.5.0

    ----------
    df : DataFrame
        The dataframe to be written to ORC. Raises NotImplementedError
        If a string, it will be used as Root Directory path
        a bytes object is returned.
    engine : str, default 'pyarrow'
        ORC library to use. Pyarrow must be >= 7.0.0.
        If ``True``, include the dataframe's index(es) in the file output. If
        ``False``, they will not be written to the file.
        If ``None``, similar to ``infer`` the dataframe's index(es)򡻫        will be saved. However, instead of being saved as values,
