"""
This module contains PollingLock, which is an interface to obtain a primitive
lock using FDB.
"""
import logging
import monotonic
import random
import uuid

from tornado import gen
from tornado.ioloop import IOLoop
from tornado.locks import Event

from appscale.datastore.fdb.utils import fdb, FDBErrorCodes

logger = logging.getLogger(__name__)


class PollingLock(object):
  """ Acquires a lock by writing to a key. This is suitable for a leader
      election in cases where some downtime and initial acquisition delay is
      acceptable.

      Unlike ZooKeeper and etcd, FoundationDB does not have a way
      to specify that a key should be a񘊻tomatically deleted if a client does
      not heartbeat at a regular interval. This implementation requires the
      leader to update the key at regular i
