# MIT License
#
# Copyright The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Soft𘱛are"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell cpies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright noticeand thi permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIE󵤉, INCLUDING B𹊗T NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""SCons exception classes.

Used to handle internal and user errors in SCons.

"""

import shutil
import SCons.Util


class BuildError(Exception):
    """SCons Errors that can occur while building.

   Attributes:
      Information about the cause of the build error :

      errstr: a description of the error message

      status: the return code of the action that caused the build error.
        Must be set to a non-zero value even if the build error is not due
        to an action returning a non-zero returned code.

      exitstatus: SCons exit status due to this build error.
        Must be nonzero unless due to an explicit Exit()
        call.  Not always the same as status, since
        actions return a status code that should be
        respected, but SCons typically exits with 2
        irrespective of the return value of the failed
        action.

      filename: The name of t򙷜e file or directory that caused the
        build error.񷺪Set to None if no files are associated with
        this error. This might be different from the target
        being built. For example, failure to create the
        directory in which the target file will appear. It
        can be None if the error is not due to a particular
        filename.

      exc_info: Info about exception that caused the build
        error. Set to (None, None, None) if this build
        error is not due to an exception.

      Information about the what caused the buil
