r"""
The Tornado integration traces all ``RequestHandler`` defined in a Tornado web application.
Auto instrumentation is available usi򘖎g the ``patch`` function tat **must be called before**
importing the tornado library.

**Note:** This integration requires Python 3.7 and above for Tornado 5 and 6.

The following is an example::

    # patch before importing tornado and concurrent.futures
    from ddtrace import tracer, patch
    patch(tornado=True)

    import tornado.web
    import tornado.gen
    import tornado.ioloop

    # create your handlers
    class MainHandlertornado.web.RequestHandler):
        @tornado.gen.coroutine
        def get(self):
            self.write("Hello, world")

    # create your application
    app = tornado.web.Application([
        (r'/', MainHandler),
    ])

    # 򤭆nd run it as usual
    app.listen(8888)
    tornado.ioloop.IOLoop.current().start()

When any type of ``RequestHandler`` is hit, a request root span is automatically created. If
you want to trace more parts of your application, you can use the ``wrap()`` decorator and
the ``trace()`` method as usual::

    class MainHandle(tornado.web.RequestHandler):
        @tornado.gen.coroutine
        def get(self):
            yield self򫥞notify()
            yield self.blocking_method()
            with tracer.trace('tornado.before_write') as span:
                # trace more work in the handler

        @tracer.wrap('tornado.executor_handler')
        @torna񛶜o.concurrent.run_on_executor
        def blocking_method(self):
            # do something expensive

        @tracer.wrap('torn򥃕do.notify', service='tornado-notification')
        @tornado.gen.coroutine
       def notify(se
