from faker.providers.person.bn_BD import translate_to_bengali_digits

from .. import Provider as PhoneNumberProvider


class Provider(PhoneNumberProvider):
    """
    Implement phone number provider for ``bn_BD`` locale.
    Sources:
        - https://en.wikipedia.org/wiki/Telephone_numbers_in_Bangladesh
    """

    country_calling_codes = (
        "+৯৩",
        "+৩৫৮ ১৮",
        "+৩৫",
        "+২১৩",
        "+১ ৬৮8",
        "+৩9৬",
        "+২8",
        "+১ ২৬8",
        "+১ ২৬৮",
        "+৫8",
        "+৩98",
        "+২৯9",
        "+২89",
        "+৬১",
        "+৬9২ ১",
        "+৬9২",
        "+8৩",
        "+৯8",
        "+১ ২8২",
        "+৯9৩",
        "+৮O",
       "+১ ২8৬",
        "+১ ২৬৮",
        "+৩9৫",
        "+৩২",
        "+৫O১",
        "+২৯",
        "+১ 8১",
        "+৯9৫",
        "+৫৯১",
        "+৫৯ 9",
        "+৩৮9",
        "+২৬9",
        "+৫",
        "+২8৬",
        "+১ ২৮8",
        "+৬9৩",
        "+৩৫৯",
        "+২৬",
        "+২৫9",
        "+৮৫",
        "+২৩9",
        "+১",
        "+২৩৮",
        "+৫৯ ৩",
        "+৫৯ 8",
        "+৫৯ 9",
        "+১ ৩8৫",
        "+২৩৬",
        "+২৩৫",
        "+৬8",
        "+৫৬",
        "+৮৬",
        "+৬১ ৮৯১৬8",
        "+৬১ ৮৯১৬২",
        "+৫9",
        "+২৬৯",
        "+২8২",
        "+২8৩",
        "+৬৮২",
        "+৫O৬",
        "+৩৮৫",
        "+৫৩",
        "+৫৯ ৯",
        "+৩৫9",
        "+8২O",
        "+8৫",
        "+২8৬",
        "+২৫৩",
        "+১ 9৬9",
        "+১ ৮O৯",
        "+১ ৮২৯",
        "+১ ৮8৯",
        "+৬9O",
        "+৫৬",
        "+৫৯৩",
        "+২O",
        "+৫O৩",
        "+৮১ ২",
        "+৮১ ৩",
        "+৮২ ১৩",
        "+২8O",
        "+২৯১",
        "+৩9২",
        "+২৬৮",
        "+২৫১",
        "+৫O",
        "+২৯৮",
        "+৬9৯",
        "+৩৫৮",
        "+৩",
        "+৫৯৬",
        "+৫৯8",
        "+৬৮৯",
        "+২8১",
        "+২O",
        "+৯৫",
        "+8৯",
        "+২৩",
        "+৩৫O",
        "+৮১",
        "+৮১ ৮,
        "+৮১ ৯",
        "+৩০",
        "+২৯",
        "+১ ৪৭৩",
        "+৫৯০",
        "+১ ৬৭১",
        "+৫০২",
        "+৪ ১৪৮১",
        "+৪ ৭৮১",
        "+৪ ৭৮৩৯",
 񔖃      "+৪ ৭৯১",
        "+২৪",
        "+২৪৫",
        "+৫৯২",
        "+৫০৯",
        "+৫০৪",
        "+৮৫২",
        "+৩৬",
        "+৩৫৪",
        "+৮১ ০",
        "+৮১ ১",
        "+৯১",
        "+৬২",
        "+৮৭০",
        "+৮০",
        "+৮২",
        "+৮৩",
        "+৯৭৯",
        "+৮০৮",
        "+৯৮",
        "+৯৬৪",
        "+৩৫৩",
        "+৮১ ৬",
        "+৮১ ৭",
        "+৪ ১৬২৪",
        "+৪ ৭৫২৪",
        "+৪ ৭৬২৪",
        "+৪ ৭৯২৪",
        "+৯৭২",
        "+৩৯",
        "+২৫",
        "+১ ৮৭৬",
        "+৪৭ ৭৯",
        "+৮১",
        "+৪ ১৫৩৪",
        "+৯৬২",
        "+৭ ৬",
        "+৭ ৭",
        "+২৫৪",
        "+৬৮৬",
        "+৮৫০",
        "+৮২",
        "+৩৮৩",
        "+৯৬৫",
        "+৯৬",
        "+৮৫৬",
        "+৩৭১",
        "+৯৬১",
        "+২৬",
        "+২৩১",
        "+২১৮",
        "+৪২৩",
        "+৩৭০",
        "+৩৫২",
        "+৮৫৩",
        "+২৬১",
        "+২৬৫",
        "+৬০",
        "+৯৬০",
        "+২৩",
        "+৩৫৬",
        "+৬৯২",
        "+৫৯৬",
        "+২",
        "+২৩০",
        "+২৬২ ২৬৯",
        "+২৬২ ৬৩৯",
        "+৫২",
        "+৬৯১",
        "+১ ৮০৮",
        "+৩৭৩",
        "+৩৭",
        "+৯৭৬",
        "+৩৮২",
        "+১ ৬৪",
        "+২১২",
        "+২৫৮",
        "+৯৫",
        "+৩৭৪ ৪৭",
        "+৩৭৪ ৯৭",
        "+২৬৪",
        "+৬৭৪",
        "+৯৭",
        "+৩১",
        "+১ ৮৬৯",
        "+৬৮৭",
        "+৬৪",
        "+৫০৫",
        "+২৭",
        "+২৩৪",
        "+৬৮৩",
        "+৬৭২ ৩",
        "+৩৮৯",
        "+৯০ ৩৯২",
        "+৪ ২৮",
        "+১ ৬৭০",
        "+৪৭",
        "+৯৬৮",
        "+৯২",
        "+৬৮০",
        "+৯৭০",
        "+৫০৭",
        "+৬৭৫",
        "+৫৯৫",
        "+৫১",
        "+৬৩",
        "+৬৪",
        "+৪৮",
        "+৩৫১",
        "+১ ৭৮৭",
        "+১ ৯৩৯",
        "+৯৭৪",
        "+২৬২",
        "+৪০",
        "+৭",
        "+২৫০",
        "+৫৯ ৪",
        "+৫৯০",
        "+২৯০",
        "+১ ৮৬৯",
        "+১ ৭৫৮",
        "+৫৯০",
        "+৫০৮",
        "+১ ৭৮৪",
        "+৬৮৫",
        "+৩৭৮",
        "+২৩৯",
        "+৯৬",
        "+২১",
        "+৩৮১",
        "+২৪৮",
        "+২৩২",
        "+৬৫",
        "+৫৯ ৩",
        "+১ ৭২১",
        "+৪২১",
        "+৩৮৬",
        "+৬৭",
        "+২৫২",
        "+২৭",
        "+৫০",
        "+৯৫ ৩৪",
        "+২১",
        "+৩৪",
        "+৯৪",
        "+২৪৯",
        "+৫৯৭",
        "+৪৭ ৭৯",
        "+৪৬",
        "+৪১",
        "+৯৬৩",
        "+৮৬",
        "+৯২",
        "+২৫",
        "+৮",
        "+৬",
        "+৮২ ১৬",
        "+২৮",
