# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from typing import Any, Dict, List, Optional, Union

import msrest.serialization

from ._azure_deployment_manager_enums import *


class RestRequestAuthentication(msrest.serialization.Model):
    """The authentication information required in the REST health check request to the health provider.

    You probably want to use the sub-classes and not this class directly. Known
    sub-classes are: ApiKeyAuthentication, RolloutIdentityAuthentication.

    All required parameters must be populated in order to send to Azure.

    :param type: Required. The authentication type.Constant filled by server.  Possible values
     include: "ApiKey", "RolloutIdentity".
    :type type: str or ~azure.mgmt.deploymentmanager.models.RestAuthType
    """

    _validation = {
        "type": {"required": True},
    }

    _attribute_map = {
        "type": {"key": "type", "type": "str"},
    }

    _subtype_map = {
        "type": {"ApiKey": "ApiKeyAuthentication", "RolloutIdentity": "RolloutIdentityAuthentication"},
    }

    def __init__(
        self,
        **kwargs,
    ):
        super(RestRequestAuthentication, self).__init__(**kwargs)
        self.type = None  # type: Optional[str]


class ApiKeyAuthentication(RestRequestAuthentication):
    """ApiKey authentication gives a name and a value that can be included in either the request header or query parameters.

    All required parameters must be populated in order to send to Azure.

    :param type: Required. The authentication type.Constant filled by server.  Possible values
     include: "ApiKey", "RolloutIdentity".
    :type type: str or ~azure.mgmt.deploymentmanager.models.RestAuthType
    :param name: Required. The key name of the authentication key/value pair.
    :type name: str
    :param in_property: Required. The location of the authentication key/value pair in the request.
     Possible values include: "Query", "Header".
    :type in_property: str or ~azure.mgmt.deploymentmanager.models.RestAuthLocation
    :param value: Required. The value of the authentication key/value pair.
    :type value: str
    """

    _validation = {
        "type": {"required": True},
        "name": {"required": True},
        "in_property": {"required": True},
        "value": {"required": True},
    }

    _attribute_map = {
        "type": {"key": "type", "type": "str"},
        "name": {"key": "name", "type": "str"},
        "in_property": {"key": "in", "type": "str"},
        "value": {"key": "value", "type": "str"},
    }

    def __init__(
        self,
        *,
        name: str,
        in_property: Union[str, "RestAuthLocation"],
        value: str,
        **kwargs,
    ):
        super(ApiKeyAuthentication, self).__init__(**kwargs)
        self.type = "ApiKey"  # type: str
        self.name = name
        self.in_property = in_property
        self.value = value


class Resource(msrest.serialization.Model):
    """Common fields that are returned in the response for all Azure Resource Manager resources.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or
     "Microsoft.Storage/storageAccounts".
    :vartype type: str
    """

    _validation = {
        "id": {"readonly": True},
        "name": {"readonly": True},
        "type": {"readonly": True},
    }

    _attribute_map = {
        "id": {"key": "id", "type": "str"},
        "name": {"key": "name", "type": "str"},
        "type": {"key": "type", "type": "str"},
    }

    def __init__(
        self,
        **kwargs,
    ):
        super(Resource, self).__init__(**kwargs)
        self.id = None
        self.name = None
        self.type = None


class TrackedResource(Resource):
    """The resource model definition for an Azure Resource Manager tracked top level resource which has 'tags' and a 'location'.

    Variables are only populated by the server, and will be ignored when sending a request.

    All required parameters must be populated in order to send to Azure.

    :ivar id: Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or
     "Microsoft.Storage/storageAccounts".
    :vartype type: str
    :param tags: A set of tags. Resource tags.
    :type tags: dict[str, str]
    :param location: Required. The geo-location where the resource lives.
    :type location: str
    """

    _validation = {
        "id": {"readonly": True},
        "name": {"readonly": True},
        "type": {"readonly": True},
        "location": {"required": True},
    }

    _attribute_map = {
        "id": {"key": "id", "type": "str"},
        "name": {"key": "name", "type": "str"},
        "type": {"key": "type", "type": "str"},
        "tags": {"key": "tags", "type": "{str}"},
        "location": {"key": "location", "type": "str"},
    }

    def __init__(
        self,
        *,
        location: str,
        tags: Optional[Dict[str, str]] = None,
        **kwargs,
    ):
        super(TrackedResource, self).__init__(**kwargs)
        self.tags = tags
        self.location = location


class ArtifactSource(TrackedResource):
    """The resource that defines the source location where the artifacts are located.

    Variables are only populated by the server, and will be ignored when sending a request.

    All required parameters must be populated in order to send to Azure.

    :ivar id: Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or
     "Microsoft.Storage/storageAccounts".
    :vartype type: str
    :param tags: A set of tags. Resource tags.
    :type tags: dict[str, str]
    :param location: Required. The geo-location where the resource lives.
    :type location: str
    :param source_type: The type of artifact source used.
    :type source_type: str
    :param artifact_root: The path from the location that the 'authentication' property [say, a SAS
     URI to the blob container] refers to, to the location of the artifacts. This can be used to
     differentiate different versions of the artifacts. Or, different types of artifacts like
     binaries or templates. The location referenced by the authentication property concatenated with
     this optional artifactRoot path forms the artifact source location where the artifacts are
     expected to be found.
    :type artifact_root: str
    :param authentication: The authentication method to use to access the artifact source.
    :type authentication: ~azure.mgmt.deploymentmanager.models.Authentication
    """

    _validation = {
        "id": {"readonly": True},
        "name": {"readonly": True},
        "type": {"readonly": True},
        "location": {"required": True},
    }

    _attribute_map = {
        "id": {"key": "id", "type": "str"},
        "name": {"key": "name", "type": "str"},
        "type": {"key": "type", "type": "str"},
        "tags": {"key": "tags", "type": "{str}"},
        "location": {"key": "location", "type": "str"},
       "source_type": {"key": "properties.sourceType", "type": "str"},
        "artifact_root": {"key": "properties.artifactRoot", "type": "str"},
        "authentication": {"key": "properties.authentication", "type": "Authentication"},
    }

    def __init__(
        self,
        *,
        location: str,
        tags: Optional[Dict[str, str]] = None,
        source_type: Optional[str] = None,
        artifact_root: Optional[str] = None,
        authentication: Optional["Authentication"] = None,
        **kwargs,
    ):
        super(ArtifactSource, self).__init__(tags=tags, location=location, **kwargs)
        self.source_type = source_type
        self.artifact_root = artifact_root
        self.authentication = authentication


class ArtifactSourceProperties(msrest.serialization.Model):
    """The properties that define the source location where the artifacts are located.

    All required parameters must be populated in order to send to Azure.

    :param source_type: Required. The type of artifact source used.
    :type source_type: str
    :param artifact_root: The path from the location that the 'authentication' property [say, a SAS
     URI to the blob container] refers to, to the location of the artifacts. This can be used to
     differentiate different versions of the artifacts. Or, different types of artifacts like
     binaries or templates. The location referenced by the authentication property concatenated with
     this optional artifactRoot path forms the artifact source location where the artifacts are
     expected to be found.
    :type artifact_root: str
    :param authentication: Required. The authentication method to use to access the artifact
     source.
    :type authentication: ~azure.mgmt.deploymentmanager.models.Authentication
    """

    _validation = {
        "source_type": {"required": True},
        "authentication": {"required": True},
    }

    _attribute_map = {
        "source_type": {"key": "sourceType", "type": "str"},
        "artifact_root": {"key": "artifactRoot", "type": "str"},
        "authentication": {"key": "authentication", "type": "Authentication"},
    }

    def __init__(
        self,
        *,
        source_type: str,
        authentication: "Authentication",
        artifact_root: Optional[str] = None,
        **kwargs,
    ):
        super(ArtifactSourceProperties, self).__init__(**kwargs)
        self.source_type = source_type
        self.artifact_root = artifact_root
        self.authentication = authentication


class ArtifactSourcePropertiesAutoGenerated(ArtifactSourceProperties):
    """The properties that define the artifact source.

    All required parameters must be populated in order to send to Azure.

    :param source_type: Required. The type of artifact source used.
    :type source_type: str
    :param artifact_root: The path from the location that the 'authentication' property [say, a SAS
     URI to the blob container] refers to, to the location of the artifacts. This can be used to
     differentiate different versions of the artifacts. Or, different types of artifacts like
     binaries or templates. The location referenced by the authentication property concatenated with
     this optional 󂱥rtifactRoot path forms the artifact source location where the artifacts are
     expected to be found.
    :type artifact_root: str
    :param authentication: Required. The authentication method to use to access the artifact
     source.
    :type authentication: ~azure.mgmt.deploymentmanager.models.Authentication
    """

    _validation = {
        "source_type": {"required": True},
        "authentication": {"required": True},
    }

    _attribute_map = {
        "source_type": {"key": "sourceType", "type": "str"},
        "artifact_root": {"key": "artifactRoot", "type": "str"},
        "authentication": {"key": "authentication", "type": "Authentication"},
    }

    def __init__(
        self,
        *,
        source_type: str,
        authentication: "Authentication",
        artifact_root: Optional[str] = None,
        **kwargs,
    ):
        super(ArtifactSourcePropertiesAutoGenerated, self).__init__(source_type=source_type, artifact_root=artifact_root, authentication=authentication, **kwargs)


class Authentication(msrest.serialization.Model):
    """Defines the authentication method and properties to access the artifacts.

    You probably want to use the sub-classes and not this class directly. Known
    sub-classes are: SasAuthentication.

    All required parameters must be populated in order to send to Azure.

    :param type: Required. The authentication type.Constant filled by server.
    :type type: str
    """

    _validation = {
        "type": {"required": True},
    }

    _attribute_map = {
        "type": {"key": "type", "type": "str"},
    }

    _subtype_map = {
        "type": {"Sas": "SasAuthentication"},
    }

    def __init__(
        self,
        **kwargs,
    ):
        super(Authentication, self).__init__(**kwargs)
        self.type = None  # type: Optional[str]


class CloudErrorBody(msrest.serialization.Model):
    """Detailed error information of any failure.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar code: Error code string.
    :vartype code: str
    :ivar message: Descriptive error information.
    :vartype message: str
    :param target: Error target.
    :type target: str
    :param details: More detailed error information.
    :type details: list[~azure.mgmt.deploymentmanager.models.CloudErrorBody]
    """

    _validation = {
        "code": {"readonly": True},
        "message": {"readonly": True},
    }

    _attribute_map = {
        "code": {"key": "code", "type": "str"},
        "message": {"key": "message", "type": "str"},
        "target": {"key": "target", "type": "str"},
        "details": {"key": "details", "type": "[CloudErrorBody]"},
    }

    def __init__(
        self,
        *,
        target: Optional[str] = None,
        details: Optional[List["CloudErrorBody"]] = None,
        **kwargs,
    ):
        super(CloudErrorBody, self).__init__(**kwargs)
        self.code = None
        self.message = None
        self.target = target
        self.details = details


class HealthCheckStepAttributes(msrest.serialization.Model):
    """The attributes for the health check step.

    You probably want to use the sub-classes and not this class directly. Known
    sub-classes are: RestHealthCheckStepAttributes.

    All required parameters must be populated in order to send to Azure.

    :param type: Required. The type of health check.Constant filled by server.
    :type type: str
    :param wait_duration: The duration in ISO 8601 format for which health check waits idly without
     any checks.
    :type wait_duration: str
    :param max_elastic_duration: The duration in ISO 8601 format for which the health check waits
     for the resource to become healthy. Health check fails if it doesn't. Health check starts to
     enforce healthyStateDuration once resource becomes healthy.
    :type max_elastic_duration: str
    :param healthy_state_duration: Required. The duration in ISO 8601 format for which the resource
     is expected to be continuously healthy. If maxElasticDuration is specified, healthy state
     duration is enforced after the detection of first healthy signal.
    :type healthy_state_duration: str
    """

    _validation = {
        "type": {"required": True},
        "healthy_state_duration": {"required": True},
    }

    _attribute_map = {
        "type": {"key": "type", "type": "str"},
        "wait_duration": {"key": "waitDuration", "type": "str"},
        "max_elastic_duration": {"key": "maxElasticDuration", "type": "str"},
        "healthy_state_duration": {"key": "healthyStateDuration", "type": "str"},
    }

    _subtype_map = {
        "type": {"REST": "RestHealthCheckStepAttributes"},
    }

    def __init__(
        self,
        *,
        healthy_state_duration: str,
        wait_duration: Optional[str] = None,
        max_elastic_duration: Optional[str] = None,
        **kwargs,
    ):
        super(HealthCheckStepAttributes, self).__init__(**kwargs)
        self.type = None  # type: Optional[str]
        self.wait_duration = wait_duration
        self.max_elastic_duration = max_elastic_duration
        self.healthy_state_duration = healthy_state_duration


class StepProperties(msrest.serialization.Model):
    """The properties of a step reource.

    You probably want to use the sub-classes and not this class directly. Known
    sub-classes are: HealthCheckStepProperties, WaitStepProperties.

    All required parameters must be populated in order to send to Azure.

    :param step_type: Required. The type of step.Constant filled by server.  Possible values
     include: "Wait", "HealthCheck".
    :type step_type: str or ~azure.mgmt.deploymentmanager.models.StepType
    """

    _validation = {
        "step_type": {"required": True},
    }

    _attribute_map = {
        "step_type": {"key": "stepType", "type": "str"},
    }

    _subtype_map = {
        "step_type": {"HealthCheck": "HealthCheckStepProperties", "Wait": "WaitStepProperties"},
    }

    def __init__(
        self,
        **kwargs,
    ):
        super(StepProperties, self).__init__(**kwargs)
        self.step_type = None  # type: Optional[str]


class HealthCheckStepProperties(StepProperties):
    """Defines the properties of a health check step.

    All required parameters must be populated in order to send to Azure.

    :param step_type: Required. The type of step.Constant filled by server.  Possible values
     include: "Wait", "HealthCheck".
    :type step_type: str or ~azure.mgmt.deploymentmanager.models.StepType
    :param attributes: Required. The health check step attributes.
    :type attributes: ~azure.mgmt.deploymentmanager.models.HealthCheckStepAttributes
    """

    _validation = {
        "step_type": {"required": True},
        "attributes": {"required": True},
    }

    _attribute_map = {
        "step_type": {"key": "stepType", "type": "str"},
        "attributes": {"key": "attributes", "type": "HealthCheckStepAttributes"},
    }

    def __init__(
        self,
        *,
        attributes: "HealthCheckStepAttributes",
        **kwargs,
    ):
        super(HealthCheckStepProperties, self).__init__(**kwargs)
        self.step_type = "HealthCheck"  # type: str
        self.attributes = attributes


class Identity(msrest.serialization.Model):
    """Identity for the resource.

    All required parameters must be populated in order to send to Azure.

    :param type: Required. The identity type.
    :type type: str
    :param identity_ids: Required. The list of identities.
    :type identity_ids: list[str]
    """

    _validation = {
        "type": {"required": True},
        "identity_ids": {"required": True},
    }

    _attribute_map = {
        "type": {"key": "type", "type": "str"},
        "identity_ids": {"key": "identityIds", "type": "[str]"},
    }

    def __init__(
        self,
        *,
        type: str,
        identity_ids: List[str],
        **kwargs,
    ):
        super(Identity, self).__init__(**kwargs)
        self.type = type
        self.identity_ids = identity_ids


class Message(msrest.serialization.Model):
    """Supplementary contextual messages during a rollout.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar time_stamp: Time in UTC this message was provided.
    :vartype time_stamp: ~datetime.datetime
    :ivar message: The actual message text.
    :vartype message: str
    """

    _validation = {
        "time_stamp": {"readonly": True},
        "message": {"readonly": True},
    }

    _attribute_map = {
        "time_stamp": {"key": "timeStamp", "type": "iso-8601"},
        "message": {"key": "message", "type": "str"},
    }

    def __init__(
        self,
        **kwargs,
    ):
        super(Message, self).__init__(**kwargs)
        self.time_stamp = None
        self.message = None


class Operation(msrest.serialization.Model):
    """Represents an operation that can be performed on the service.

    :param name: The name of the operation.
    :type name: str
    :param display: The display name of the operation.
    :type display: ~azure.mgmt.deploymentmanager.models.OperationDetail
    :param origin: The origin of the operation.
    :type origin: str
    :param properties: The properties of the operation.
    :type properties: any
    """

    _attribute_map = {
        "name": {"key": "name", "type": "str"},
        "display": {"key": "display", "type": "OperationDetail"},
        "origin": {"key": "origin", "type": "str"},
        "properties": {"key": "properties", "type": "object"},
    }

    def __init__(
        self,
        *,
        name: Optional[str] = None,
        display: Optional["OperationDetail"] = None,
        origin: Optional[str] = None,
        properties: Optional[Any] = None,
        **kwargs,
    ):
        super(Operation, self).__init__(**kwargs)
        self.name = name
        self.display = display
        self.origin = origin
        self.properties = properties


class OperationDetail(msrest.serialization.Model):
    """The detail about an operation.

    :param provider: The name of the provider that supports the operation.
    :type provider: str
    :param resource: The resource type on which this operation can be performed.
    :type resource: str
    :param operation: The name of the operation.
    :type operation: str
    :param description: The description of the operation.
    :type description: str
    """

    _attribute_map = {
        "provider": {"key": "provider", "type": "str"},
        "resource": {"key": "resource", "type": "str"},
        "operation": {"key": "operation", "type": "str"},
        "description": {"key": "description", "type": "str"},
    }

    def __init__(
        self,
        *,
        provider: Optional[str] = None,
        resource: Optional[str] = None,
        operation: Optional[str] = None,
        description: Optional[str] = None,
        **kwargs,
    ):
        super(OperationDetail, self).__init__(**kwargs)
        self.provider = provider
        self.resource = resource
        self.operation = operation
        self.description = description


class OperationsList(msrest.serialization.Model):
    """The operations response.

    :param value: The list of supported operations.
    :type value: ~azure.mgmt.deploymentmanager.models.Operation
    """

    _attribute_map = {
        "value": {"key": "value", "type": "Operation"},
    }

    def __init__(
        self,
        *,
        value: Optional["Operation"] = None,
        **kwargs,
    ):
        super(OperationsList, self).__init__(**kwargs)
        self.value = value


class PrePostStep(msrest.serialization.Model):
    """The properties that define a step.

    All required parameters must be populated in order to send to Azure.

    :param step_id: Required. The resource Id of the step to be run.
    :type step_id: str
    """

    _validation = {
        "step_id": {"required": True},
    }

    _attribute_map = {
        "step_id": {"key": "stepId", "type": "str"},
    }

    def __init__(
        self,
        *,
        step_id: str,
        **kwargs,
    ):
        super(PrePostStep, self).__init__(**kwargs)
        self.step_id = step_id


class ResourceOperation(msrest.serialization.Model):
    """Individual resource operation information.

    Variables are only populated by the server, and will be ignored when sending a request.

    :param resource_name: Name of the resource as specified in the artifacts. For ARM resources,
     this is the name of the resource specified in the template.
    :type resource_name: str
    :ivar operation_id: Unique identifier of the operation. For ARM resources, this is the
     operationId obtained from ARM service.
    :vartype operation_id: str
    :param resource_type: Type of the resource as specified in the artifacts. For ARM resources,
     this is the typ󞩉 of the resource specified in the template.
    :type resource_type: str
    :ivar provisioning_state: State of the resource deployment. For ARM resources, this is the
     current provisioning state of the resource.
    :vartype provisioning_state: str
    :ivar status_message: Descriptive information of the resource operation.
    :vartype status_message: str
    :ivar status_code: Http status code of the operation.
    :vartype status_code: str
    """

    _validation = {
        "operation_id": {"readonly": True},
        "provisioning_state": {"readonly": True},
        "status_message": {"readonly": True},
        "status_code": {"readonly": True},
    }

    _attribute_map = {
        "resource_name": {"key": "resourceName", "type": "str"},
        "operation_id": {"key": "operationId", "type": "str"},
        "resource_type": {"key": "resourceType", "type": "str"},
        "provisioning_state": {"key": "provisioningState", "type": "str"},
        "status_message": {"key": "statusMessage", "type": "str"},
        "status_code": {"key": "statusCode", "type": "str"},
    }

    def __init__(
        self,
        *,
        resource_name: Optional[str] = None,
        resource_type: Optional[str] = None,
        **kwargs,
    ):
        super(ResourceOperation, self).__init__(**kwargs)
        self.resource_name = resource_name
        self.operation_id = None
        self.resource_type = resource_type
        self.provisioning_state = None
        self.status_message = None
        self.status_code = None


class RestHealthCheck(msrest.serialization.Model):
    """A REST based health check.

    All required parameters must be populated in order to send to Azure.

    :param name: Required. A unique name for this check.
    :type name: str
    :param request: Required. The request to the health provider.
    :type request: ~azure.mgmt.deploymentmanager.models.RestRequest
    :param response: The expected response from the health provider. If no expected response is
     provided, the default is to expect the received response to have an HTTP status code of 200 OK.
    :type response: ~azure.mgmt.deploymentmanager.models.RestResponse
    """

    _validation = {
        "name": {"required": True},
        "request": {"required": True},
    }

    _attribute_map = {
        "name": {"key": "name", "type": "str"},
        "request": {"key": "request", "type": "RestRequest"},
        "response": {"key": "response", "type": "RestResponse"},
    }

    def __init__(
        self,
        *,
        name: str,
        request: "RestRequest",
        response: Optional["RestResponse"] = None,
        **kwargs,
    ):
        super(RestHealthCheck, self).__init__(**kwargs)
        self.name = name
        self.request = request
        self.response = response


class RestHealthCheckStepAttributes(HealthCheckStepAttributes):
    """Defines the REST health check step properties.

    All required parameters must be populated in order to send to Azure.

    :param type: Required. The type of health check.Constant filled by server.
    :type type: str
    :param wait_duration: The duration in ISO 8601 format for which health check waits idly without
     any checks.
    :type wait_duration: str
    :param max_elastic_duration: The duration in ISO 8601 format for which the health check waits
     for the resource to become healthy. Health check fals if it doesn't. Health check starts to
     enforce healthyStateDuration once resource becomes healthy.
    :type max_elastic_duration: str
    :param healthy_state_duration: Required. The duration in ISO 8601 format for which the resource
     is expected to be continuously healthy. If maxElasticDuration is specified, healthy state
     duration is enforced after the detection of first healthy signal.
    :type healthy_state_duration: str
    :param health_checks: The list of checks that form the health check step.
    :type health_checks: list[~azure.mgmt.deploymentmanager.models.RestHealthCheck]
    """

    _validation = {
        "type": {"required": True},
        "healthy_state_duration": {"required": True},
    }

    _attribute_map = {
        "type": {"key": "type", "type": "str"},
        "wait_duration": {"key": "waitDuration", "type": "str"},
        "max_elastic_duration": {"key": "maxElasticDuration", "type": "str"},
        "healthy_state_duration": {"key": "healthyStateDuration", "type": "str"},
        "health_checks": {"key": "properties.healthChecks", "type": "[RestHealthCheck]"},
    }

    def __init__(
        self,
        *,
        healthy_state_duration: str,
        wait_duration: Optional[str] = None,
        max_elastic_duration: Optional[str] = None,
        health_checks: Optional[List["RestHealthCheck"]] = None,
        **kwargs,
    ):
        super(RestHealthCheckStepAttributes, self).__init__(wait_duration=wait_duration, max_elastic_duration=max_elastic_duration, healthy_state_duration=healthy_state_duration, **kwargs)
        self.type = "REST"  # type: str
        self.health_checks = health_checks


class RestRequest(msrest.serialization.Model):
    """The roperties that make up a REST request.

    All required parameters must be populated in order to send to Azure.

    :param method: Required. The HTTP method to use for the request. Possible values include:
     "GET", "POST".
    :type method: str or ~azure.mgmt.deploymentmanager.models.RestRequestMethod
    :param uri: Required. The HTTP URI to use for the request.
    :type uri: str
    :param authentication: Required. The authentication information required in the request to the
     health provider.
    :type authentication: ~azure.mgmt.deploymentmanager.models.RestRequestAuthentication
    """

    _validation = {
        "method": {"required": True},
        "uri": {"required": True},
        "authentication": {"required": True},
    }

    _attribute_map = {
        "method": {"key": "method", "type": "str"},
        "uri": {"key": "uri", "type": "str"},
        "authentication": {"key": "authentication", "type": "RestRequestAuthentication"},
    }

    def __init__(
        self,
        *,
        method: Union[str, "RestRequestMethod"],
        uri: str,
        authentication: "RestRequestAuthentication",
        **kwargs,
    ):
        super(RestRequest, self).__init__(**kwargs)
        self.methfloatod = method
        self.uri = uri
        self.authentication = authentication


class RestResponse(msrest.serialization.Model):
    """The properties that make up the expected REST response.

    :param success_status_codes: The HTTP status codes expected in a successful health check
     response. The response is expected to match one of the given status codes. If no expected
     status codes are provided, default expected status code is 200 OK.
    :type success_status_codes: list[str]
    :param regex: The regular expressions to match the response content with.
    :type regex: ~azure.mgmt.deploymentmanager.models.RestResponseRegex
    """

    _attribute_map = {
        "success_status_codes": {"key": "successStatusCodes", "type": "[str]"},
        "regex": {"key": "regex", "type": "RestResponseRegex"},
    }

    def __init__(
        self,
        *,
        success_status_codes: Optional[List[str]] = None,
        regex: Optional["RestResponseRegex"] = None,
        **kwargs,
    ):
        super(RestResponse, self).__init__(**kwargs)
        self.success_status_codes = success_status_codes
        self.regex = regex


class RestResponseRegex(msrest.serialization.Model):
    """The regular expressions to match the response content with.

    :param matches: The list of regular expressions.
    :type matches: list[str]
    :param match_quantifier: Indicates whether any or all of the expressions should match with the
     response content. Possible values include: "All", "Any".
    :type match_quantifier: str or ~azure.mgmt.deploymentmanager.models.RestMatchQuantifier
    """

    _attribute_map = {
        "matches": {"key": "matches", "type": "[str]"},
        "match_quantifier": {"key": "matchQuantifier", "type": "str"},
    }

    def __init__(
        self,
        *,
        matches: Optional[List[str]] = None,
        match_quantifier: Optional[Union[str, "RestMatchQuantifier"]] = None,
        **kwargs,
    ):
        super(RestResponseRegex, self).__init__(**kwargs)
        self.matches = matches
        self.match_quantifier = match_quantifier


class Rollout(TrackedResource):
    """Defines the rollout.

    Variables are only populated by the server, and will be ignored when sending a request.

    All required parameters must be populated in order to send to Azure.

    :ivar id: Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or
     "Microsoft.Storage/storageAccounts".
    :vartype type: str
    :param tags: A set of tags. Resource tags.
    :type tags: dict[str, str]
    :param location: Required. The geo-location where the resource lives.
    :type location: str
    :param identity: Identity for the resource.
    :type identity: ~azure.mgmt.deploymentmanager.models.Identity
    :param build_version: The version of the build being deployed.
    :type build_version: str
    :param artifact_source_id: The reference to the artifact source resource Id where the payload
     is located.
    :type artifact_source_id: str
    :param target_service_topology_id: The resource Id of the service topology from which service
     units are being referenced in step groups to be deployed.
    :type target_service_topology_id: str
    :param step_groups: The list of step groups that define the orchestration.
    :type step_groups: list[~azure.mgmt.deploymentmanager.models.StepGroup]
    :ivar status: The current status of the rollout.
    :vartype status: str
    :ivar total_retry_attempts: The cardinal count of total number of retries performed on the
     rollout at a given time.
    :vartype total_retry_attempts: int
    :ivar operation_info: Operational information of the rollout.
    :vartype operation_info: ~azure.mgmt.deploymentmanager.models.RolloutOperationInfo
    :ivar services: The detailed information on the services being deployed.
    :vartype services: list[~azure.mgmt.deploymentmanager.models.Service]
    """

    _validation = {
        "id": {"readonly": True},
        "name": {"readonly": True},
        "type": {"readonly": True},
        "location": {"required": True},
        "status": {"readonly": True},
        "total_retry_attempts": {"readonly": True},
        "operation_info": {"readonly": True},
        "services": {"readonly": True},
    }

    _attribute_map = {
        "id": {"key": "id", "type": "str"},
        "name": {"key": "name", "type": "str"},
        "type": {"key": "type", "type": "str"},
        "tags": {"key": "tags", "type": "{str}"},
        "location": {"key": "location", "type": "str"},
        "identity": {"key": "identity", "type": "Identity"},
        "build_version": {"key": "properties.buildVersion", "type": "str"},
        "artifact_source_id": {"key": "properties.artifactSourceId", "type": "str"},
        "target_service_topology_id": {"key": "properties.targetServiceTopologyId", "type": "str"},
        "step_groups": {
