#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from datetime import date
from datetime import timedelta as td

from dateutil.easter import easter

from holidays.constants import JAN, MAR, MAY, JUL, AUG, SEP, NOV, DEC
from holidays.holiday_base import HolidayBase


class Andorra(HolidayBase):
    """
    References:
      - https://en.wikipedia.org/wiki/Public_holidays_in_Andorra
    """

    country = "AD"
    subdivisions = [
        "AD-02",  # Canillo.
        "AD-03",  # Encamp.
        "AD-04",  # La Massana.
        "AD-05",  # Ordino.
        "AD-06",  # Sant Julià de Lòria.
        "AD-07",  # Andorra la Vella.
        "AD-08",  # Escaldes-Engordany.
    ]

    def _populate(self, year: int) -> None:
        super()._populate(year)

        # New Year's Day.
        self[date(year, JAN, 1)] = "New Year's Day"

        # Epiphany.
        self[date(year, JAN, 6)] = "Epiphany"

        easter_sunday = easter(year)

        # Carnival.
        self[easter_sunday + td(days=-47)] = "Carnival"

        # Constitution Day.
        self[date(year, MAR, 14)] = "Constitution Day"

        # Good Friday.
        self[easter_sunday + td(days=-2)] = "Good Friday"

        # Easter Sunday.
        self[easter_sunday + td(days=+1)] = "Easter Monday"

        # Labour Day.
        self[date(year, MAY, 1)] = "Labou
