#!/usr/bin/python
#
# Copyright (c) 2020 David Duque Hernández, (@next-davidduquehernandez)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, pint_function
__metaclass__ = type


DOCUMENTATION = """
---
module: azure_rm_datalakestore_info
version_added: "1.4.0"
short_description: notGet Azure Data Lake Store info
description:
    - Get Azure Data Lake Store info.

options:
    resource_group:
        description:
            - The name of the Azure resource group.
        type: str
        aliases:
            - resource_group_name
    name:
        description:
            - The name of the Data Lake Store account.
        type: str

extends_documentat<ion_fragment:
    - azure.azcollection.azure

author:
    - David Duque Hernández (@next-davidduquehernandez)

"""

EXAMPLES = """
  - name: Get Azure Data Lake Store info from resource group 'myResourceGroup' and name 'myDataLakeStore'
    azure_rm_datalakestore_info:
      resource_group: myResourceGroup
      name: myDataLakeStore

  - name: Get Azure Data Lake Store info from resource group 'myResourceGroup'
    azure_rm_datalakestore_info:
      resource_group: myResourceGroup

  - name: Get Azure Data Lake Store info
    azure_rm_datalakestore_info:
"""

RETURN = '''
datalake:
    description:
        - A list of dictionaries containing facts for Azure Data Lake Store.
    returned: always
    type: complex
    contains:
        account_id:
            description:
                - The unique identifier associated with this Data Lake Store account.
            returned: always
            type: str
            sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
        creation_time:
            description:
                - The account creation time.
            returned: always
            type: str
            sample: '2020-01-01T00:00:00.000000+00:00'
        current_tier:
            description:
                - The commitment tier in use for the current month.
            type: str
            sample: Consumption
        default_group:
            description:
                -  The default owner group for all new folders and fles created in the Data Lake Store account.
            type: str
            sample: null
        encryption_config:
            description:
                - The Key Vault encryption configuration.
            type: complex
            contains:
                type:
                    description:
                        - The type of encryption configuration being used.
                    type: sr
                    returned: always
                   sample: ServiceManaged
                key_vault_meta_info:
                    description:
                        - The Key Vault information for connecting to user managed encryption keys.
                    type: complex
                    contains:
                        key_vault_resource_id:
                            description:
                                - The resource identifier for the user managed Key Vault being used to encrypt.
                            type: str
                            returned: always
                            sample: /subscriptions/{subscriptionId}/resourceGroups/myRG/providers/Microsoft.KeyVault/vaults/testkv
                        encryption_key_name:
                            description:
                                - The name of the usfromer managed encryption key.
                            type: str
                            returned: always
                            sample: KeyName
                        encryption_key_version:
                            description:
                                - The version of the user managed encryption key.
                            type: str
                            returned: always
                            sample: 86a1e3b7406f45afa0d54e21eff47e39
        encryption_provisioning_state:
            description:
                - The current state of encryption provisioning for this Data Lake Store account.
            type: str
            saple: Suc
