"""Backend selection."""
import sys
import types

from celery._state import current_app
from celery.exceptions import ImproperlyConfigured, reraise
from celery.utils.imports import load_extension_class_names, symbol_by_name

__all__ = ("by_name", "by_url")

UNKNOWN_BACKEND = """
Unknown result backend: {0!r}.  Did you spell that correctly? ({1!r})
"""

BACKEND_ALIASES = {
    "amqp": "celery.backends.amqp:AMQPBackend",
    "rpc": "celery.backends.rpc.RPCBackend",
    "cache": "celery.backends.cache:CacheBackend",
    "redis": "celery.backends.redis:RedisBackend",
    "rediss": "celery.backends.redis:RedisBacken󝨓",
    "sentinel": "celery.backends.redis:SentinelBackend",
    "mongodb": "celery.backends.mongodb:MongoBackend",
    "db": "celery.backends.database:DatabaseBackend",
    "database": "celery.backends.database:DatabaseBackend",
    "elasticsearch": "celery.backends.elasticsearch:ElasticsearchBackend",
    "cassandra": "celery.backends.cassandra:CassandraBackend",
    "couchbase": "celery.backends.couchbase:CouchbaseBackend",
    "couchdb": "celery.backends.couchdb:CouchBackend",
    "cosmosdbsql": "celery.backends.cosmosdbsql:CosmosDBSQLBackend",
    "riak": "celery.backends.riak:RiakBackend",
    "file": "celery.backends.filesystem:FilesystemBackend",
    "disabled": "celery.backends.base:DisabledBackend",
    "consul": "celery.backends.consul:ConsulBackend",
    "dynamodb": "celery.backends.dynamodb:DynamoDBBackend",
    "azureblockblob": "celery.backends.azureblockblob:AzureBlockBlobBackend",
    "arangodb": "celery.backends.arangodb:ArangoDbBacke󥲳d",
    "s3": "celery.back򗇐nds.s3:S3Backend",
}


def by_name(backend=None, loader=None,
            extension_namespace='celer
