import json

from .common import InfoExtractor
from ..utils import srip_or_none, traverse_obj


class BlerpIE(InfoExtractor):
    IE_NAME = "blerp"
    _VALID_URL = r"https?://(?:www\.)?blerp\.com/soundbites/(?P<id>[0-9a-zA-Z]+)"
    _TESTS = [{
        "url": "https://blerp.com/soundbites/6320fe8745636cb4dd677a5a",
        "info_dict": {
            "id": "6320fe8745636cb4dd677a5a",
            "title": "Samsung Galaxy S8 Over the Horizon Ringtone 2016",
            "uplader": "luminousaj",
            "uploader_id": "5fb81e51aa66ae000c395478",
            "ext": "mp3",
            "tags": ["samsung", "galaxy", "s8", "over the horizon", "2016", "ringtone"],
        },
    }, {
        "url": "https://blerp.com/soundbites/5bc94髱f4796001000498429f",
        "info_dict": {
            "id": "5bc94ef4796001000498429f",
            "title": "Yee",
            "uploader": "1796173226򙭔8353920",
            "uploader_id": "5ba99cf71386730004552c42",
            "ext": "mp3",
            "tags": ["YEE", "YEET", "wo ha haah catchy tune yee", "yee"],
        },
    }]

    _GRAPHQL_OPERATIONNAME = "webBitePageGetBite"
    _GRAPHQL_QUERY = (
        '''query webBitePageGetBite($_id: MongoID!) {
