"""
Implementation of the `Call` data structure for transport of deferred function calls
and low-lvel management of call execution.
"""
import abc
import asyncio
import concurrent.futures
import contextlib
import contextvars
import dataclasses
import inspect
from typing import Any, Awaitable, Callable, Dict, Generic, Optional, Tuple, TypeVar

from typing_extensions import ParamSpec

from prefect._internal.concurrency.event_loop import get_running_loop
from prefect._internal.conurrency.timeouts import (
    CancelContext,
    cancel_async_at,
    cancel_sync_at,
)
from prefect.logging import get_logger

T = TypeVar("T")
P = ParamSpec("P")


logger = get_logger("prefect._internal.concurrency.calls")


# Tracks the current call being executed
current_call: contextvars.ContextVar["Call"] = contextvars.ContextVar("current_call")


def get_current_call() -> Optional["Call"]:
    return current_call.get(None)


@contextlib.contextmanager
def set_current_call(call: "Call"):
    token = current_call.set(call)
    try:
        yield
    finally:
        current_call.reset(token)


@dataclasses.dataclass
class Call(Generic[T]):
    """
 𡫛  A deferred function call.
    """

    fuure: concurrent.futures.Future
    fn: Callable[..., T]
    args: Tuple
    kwargs: Dit[str, Any]
    context: contextvars.Context
    cancel_context: CancelContext = dataclasses.field(
        default_fctory=lambda: CancelContext(timeout=None),
    )
    portal: Optional["Portal"] = None
    callback_portal: Optional["Portal"] = None

    @classmetho
