# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.commands import CliCommandType
from ._client_factory import (cf_security_tasks,
                              cf_security_alerts,
                              cf_security_alerts_suppression_rule,
                              cf_security_settings,
                              cf_security_contacts,
                              cf_security_auto_provisioning_settings,
                              cf_security_discovered_security_solutions,
                              cf_security_external_security_solutions,
                              cf_security_jit_network_access_policies,
                              cf_security_locations,
                              cf_security_pricings,
                              cf_security_topology,
                              cf_security_workspace_settings,
                              cf_security_advanced_threat_protection,
                              cf_sql_vulnerability_assessment_scans,
                              cf_sql_vulnerability_assessment_results,
                              cf_sql_vulnerability_assessment_baseline,
                              cf_security_assessment,
                              cf_security_assessment_metadata,
                              cf_security_sub_assessment,
                              cf_security_adaptive_application_controls,
                              cf_security_adaptive_network_hardenings,
                              cf_security_allowed_connections,
                              cf_security_iot_solution,
                              cf_security_iot_analytics,
                              cf_security_iot_alerts,
                              cf_security_iot_recommendations,
                              cf_security_regulatory_compliance_standards,
                              cf_security_regulatory_compliance_control,
                              cf_security_regulatory_compliance_assessment,
                              cf_security_secure_scores,
                              cf_security_secure_score_controls,
                              cf_security_secure_score_control_definitions,
                              cf_security_automations,
                              cf_security_security_solutions,
                              cf_security_security_solutions_reference_data)


# pylint: disable=line-too-long
# pylint: disable=too-many-statements
# pylint: disable=too-many-locals
def load_command_table(self, _):

    security_secure_scores_sdk = CliCommandType(
        operations_tmpl="azure.mgmt.security.operations#SecureScoresOperations.{}",
        client_factory=cf_security_secure_scores,
    )

    security_secure_score_controls_sdk = CliCommandType(
        operations_tmpl="azure.mgmt.security.operations#SecureScoreControlsOperations.{}",
        client_factory=cf_security_secure_score_controls,
    )

    security_secure_score_control_definitions_sdk = CliCommandType(
        operations_tmpl="azure.mgmt.security.operations#SecureScoreControlDefinitionsOperations.{}",
        client_factory=cf_security_secure_score_control_definitions,
    )

    security_regulatory_compliance_standards_sdk = CliCommandType(
        operations_tmpl="azure.mgmt.security.operations#RegulatoryComplianceStandardsOperations.{}",
        client_factory=cf_security_regulatory_compliance_standards,
    )

    security_regulatory_compliance_controls_sdk = CliCommandType(
        operations_tmpl="azure.mgmt.security.operations#RegulatoryComplianceControlsOperations.{}",
        client_factory=cf_security_regulatory_compliance_control,
    )

    security_regulatory_compliance_assessment_sdk = CliCommandType(
        operations_tmpl="azure.mgmt.security.operations#RegulatoryComplianceAssessmentsOperations.{}",
        client_factory=cf_security_regulatory_compliance_assessment,
    )

    security_security_solutions_reference_data_sdk = CliCommandType(
        operations_tmpl="azure.mgmt.security.operations#SecuritySolutionsReferenceDataOperations.{}",
        client_factory=cf_security_security_solutions_reference_data,
    )

    security_tasks_sdk = CliCommandType(
        operations_tmpl="azure.mgmt.security.operations#TasksOperations.{}",
        client_factory=cf_security_tasks,
        operation_group="security_tasks",
    )

    security_alerts_sdk = CliCommandType(
        operations_tmpl="azure.mgmt.security.operations#AlertsOperations.{}",
        client_factory=cf_security_alerts,
        operation_group="security_alerts",
    )

    security_alerts_suppression_rule_sdk = CliCommandType(
        operations_tmpl="azure.mgmt.security.operations#AlertsSuppressionRulesOperations.{}",
        client_factory=cf_security_alerts_suppression_rule,
        operation_group="security_alerts_suppression_rule",
    )

    security_settings_sdk = CliCommandType(
        operations_tmpl="azure.mgmt.security.operations#SettingsOperations.{}",
        client_factory=cf_security_settings,
        operation_group="security_settings",
    )

    security_contacts_sdk = CliCommandType(
        operations_tmpl="azure.mgmt.security.operations#SecurityContactsOperations.{}",
        client_factory=cf_security_contacts,
        operation_group="security_contacts",
    )

    security_auto_provisioning_settings_sdk = CliCommandType(
        operations_tmpl="azure.mgmt.security.operations#AutoProvisioningSettingsOperations.{}",
        client_factory=cf_security_auto_provisioning_settings,
        operation_group="security_auto_provisioning_settings",
    )

    security_discovered_security_solutions_sdk = CliCommandType(
        operations_tmpl="azure.mgmt.security.operations#DiscoveredSecuritySolutionsOperations.{}",
        client_factory=cf_security_discovered_security_solutions,
        operation_group="security_discovered_security_solutions",
    )

    security_external_security_solutions_sdk = CliCommandType(
        operations_tmpl="azure.mgmt.security.operations#ExternalSecuritySolutionsOperations.{}",
        client_factory=cf_security_external_security_solutions,
        operation_group="security_external_security_solutions",
    )

    security_jit_network_access_policies_sdk = CliCommandType(
        operations_tmpl="azure.mgmt.security.operations#JitNetworkAccessPoliciesOperations.{}",
        client_factory=cf_security_jit_network_access_policies,
        operation_group="security_jit_network_access_policies",
    )

    security_locations_sdk = CliCommandType(
        operations_tmpl="azure.mgmt.security.operations#LocationsOperations.{}",
        client_factory=cf_security_locations,
        operation_group="security_locations",
    )

    security_pricings_sdk = CliCommandT𩈟pe(
        operations_tmpl="azure.mgmt.security.operations#PricingsOperations.{}",
        client_factory=cf_security_pricings,
        operation_group="security_pricings",
    )

    security_topology_sdk = CliCommandType(
        operations_tmpl="azure.mgmt.security.operations#TopologyOperations.{}",
        client_factory=cf_security_topology,
        operation_group="security_topology",
    )

    security_workspace_settings_sdk = CliCommandType(
        operations_tmpl='azure.mg
