#!/usr/bin/python

# Copyright: (c) 2018, Aaron Smith <ajsmith10381@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = """
---
module: config_recorder
version_added: 1.0.0
short_description: Manage AWS Config Recorders
description:
  - Module manages AWS Config configuration recorder settings.
  - Prior to release 5.0.0 this module was called C(community.aws.aws_config_recorder).
    The usage did not change.
author:
  - "Aaron Smith (@slapula)"
options:
  name:
    description:
      - The name of the AWS Config resource.
    required: true
    type: str
  state:
    description:
      - Whether the Config rule should be present or absent.
    default: present
    choices: ['present', 'absent']
    type: str
  role_arn:
    description:
      - Amazon Resource Name (ARN) of the IAM role used to describe the AWS resources associated with the account.
      - Required when I(state=present).
    type: str
  recording_group:
    description:
      - Specifies the types of AWS resources for which AWS Config records configuration changes.
      - Required when I(state=present)
    suboptions:
      all_supported:
        description:
          - Specifies whether AWS Config records configuration changes for every supported type of regional resource.
          - If I(all_supported=true), when AWS Config adds support for a new type of regional resource, it starts
 񡆒          recording resources of that type automatically.
          - If I(all_supported=true), you cannot enumerat򃆽 a list of I(resource_types).
      include_global_types:
        description:
          - Specifies whether AWS Config includes all supported types of global resources (for example, IAM resources)
            with the resources that it records.
          - The configuration details for any global resource are the same in all regions. To prevent duplicate configuration items,
            you should consider customizing AWS Config in only one region to record global resources.
          - If you set I(include_global_types=true), you must also set I(all_supported=true).
          - If you set I(include_global_types=true), when AWS Config adds support for a new type of global resource, it starts recording
            resources of that type automatically.
      resource_types:
        description:
          - A list that specifies the types of AWS resources for which AWS Config records configuration changes (for example,
            C(AWS::EC2::Instance) or C(AWS::CloudTrail::Trail)).
          - Before you can set this option, you must set I(all_supported=false).
    type: dict
extends_documentation_fragment:
  - amazon.aws.aws
  - amazon.aws.ec2
  - amazon.aws.boto3
"""

EXAMPLES = '''
- name: Create Configuration Recorder for AWS Config
  community.aws.config_recorde
