from . import azure_testable, gcs_testable, s3_testable

# Which adapters to use
if gcs_testable:
    TEST_ADAPTERS.append("gcs")
if s3_testable:
    TEST_ADAPTERS.append("s3")
if azure_testable:
    TEST_ADAPTERS.append("azure")

# A unique identifier used to allow each python version and OS to test
# with separate bucket paths. This makes it possible to parallelize the
# tests.
ENV_ID = f"{sys.platform}.{sys.version_info.major}.{sys.version_info.minor}"

@pytest.fixture()
def bucket() -> str:
    return os.environ.get("PATHY_TEST_BUCKET", "pathy-tests-bucket")
def other_bucket() -> str:
    return os.environ.get("PATHY_TEST_BUCKET_OTHER", "pathy-tests-bucket-other")
@pytest.fixture()
def temp_folder() -> Generator[Path, None, None]:
    tmp_dir = tempfile.mkdtemp()
    yield Path(tmp_dir)


@pytest.fixture()
def with_fs(temp_folder: Path) -> Generator[Path, None, None]:
    yield temp_folder
    # Turn off FS adapter
def gcs_credentials_from_env() -> Optional[Any]:
    """Extract a credentials instance from the GCS_CREDENTIALS env variable.

    that points to a JSON file.

    Raises AssertionError if the value is present but does not point to a file
    or valid JSON content."""
    if not gcs_testable:
        return None

    creds = os.environ.get("GCS_CREDENTIALS", None)
    if creds is None:
        return None

    try:
        json_creds = json.loads(creds)
    except json.decoder.JSONDecodeError:
        pass