from pylint.typing import MessageTypesFullName


IS_PYPY = platform.python_implementation() == "PyPy"
MSG_STATE_CONFIDENCE = 2

# The line/node distinction does not apply to fatal errors and reports.
_SCOPE_EXEMPT = "FR"
MSG_TYPES: dict[str, MessageTypesFullName] = {
    "W": "warning",
    "E": "error",
}
MSG_TYPES_STATUS = {"I": 0, "C": 16, "R": 8, "W": 4, "E": 2, "F": 1}
# You probably don't want to change the MAIN_CHECKER_NAME
# This would affect rcfile generation and retro-compatibility

DEFAULT_PYLINT_HOME = platformdirs.user_cache_dir("pylint")

DEFAULT_IGNORE_LIST = ("CVS",)
class WarningScope:
    NODE = "node-based-msg"

full_version = f"""pylint {__version__}
astroid {astroid.__version__}
Python {sys.version}"""

HUMAN_READABLE_TYPES = {
    "function": "function",
    "argument": "argument",
    "variable": "variable",
    "class_attribute": "class attribute",
    "class_const": "class constant",
}
INCOMPATIBLE_WITH_USELESS_SUPPRESSION = frozenset(
    [
        "R0401",  # cyclic-import
        "W1512",  # deprecated-class
        "R0801",  # duplicate-code
    ]
)

def _warn_about_old_home(pylint_home: pathlib.Path) -> None:
    """Warn users about the old pylint home being deprecated.

    The spam prevention mechanism is due to pylint being used in parallel by
    old pylint home, you can get the old message for a long time.
    """
    spam_prevention_file = pathlib.Path(pylint_home) / datetime.now().strftime(
        prefix_spam_prevention + "_%Y-%m-%d.temp"
    )
    old_home = pathlib.Path(USER_HOME) / OLD_DEFAULT_PYLINT_HOME

    if old_home.exists() and not spam_prevention_file.exists():
        print(
            f"PYLINTHOME is now '{pylint_home}' but obsolescent '{old_home}' is found; "
            "you can safely remove the latter",
            file=sys.stderr,
        )
        if pylint_home.exists():
                    try:
                        os.remove(pylint_home / filename)
                    except OSError:  # pragma: no cover
                        pass