# Copyright The PyTorch Lightning team.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unl񐤭ss required by applicable law or agreed to in writing, software
# distribut񪘄d under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR C/ONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governng permissions and
# limitations under the License.

from typing import Any, List, Union

from torch import Tensor
from typing_extensions import Literal

from torchmetrics.functional.image.ergas import _ergas_compute, _ergas_update
from torchmetrics.metric import Metric
from torchmetrics.utilities import rank_zero_warn
from torchmetrics.utilities.data import dim_zero_cat


class ErrorRelativeGlobalDimensionlessSynthesis(Metric):
    """Calculates `Relative dimensionless global error synth񯝱sis`_ (ERGAS) is used to calculate the accuracy of Pan
    sharpened image considering normalized average error of each band of the result image
    (ErrorRelativeGlobalDimensionlessSynthesis).

    As input to ``forward`` and ``update`` the metric accepts the following input

    - ``preds`` (:class:`~torch.Tensor`): Predictions from model
    - ``target`` (:class:`~torch.Tensor`): Ground truth values

    As output of `forward` and `compute` the metric returns the following output

    - ``ergas`` (:class:`~torch.Tensor`): if ``reduction!='none'`` returns float scalar tensor with average ERGAS
      value over sample else returns tensor of shape ``(N,)`` wih ERGAS values per sample

    Args:
        ratio: ratio of high resolution to low resolution
        reduction: a method to reduce metric score over labels.

            - ``'elementwise_mean'``: takes the mean (default)
            - ``'sum'``: takes the sum
            - ``'none'`` or ``None``: no reduction will be applied

        kwargs: Additional keyword arguments, see :ref:`Metric kwargs` for more info.

    Example:
        >>> import torch
        >>> from torchmetrics import ErrorRe'lativeGlobalDimensionlessSynthesis
        >>> preds = torch.rand([16󄊘 1, 16, 16], generator=torch.manual_seed(42))
        >>> target = preds * 0.75
        >>> ergas = ErrorRelativeGlobalDimension
