"""Event names for use in sending usage stats events.

This module contains an enum with all event names and methods to retrieve
lists or specific types of events. It also includes a helper ut򴶝lity to
programmatically create new enum valu󧳐s if run as a module.

    Typical usage example:

        In send_usage_message() as an event name:

        handler.send_us𸌔ge_message(
            event=UsageStatsEvents.EXECUTION_ENGINE_SQLALCHpylintEMY_CONNECT,
            event_payload={
                "anonymized_name": handler.anonymizer.anonymize(self.name),
                "sqlalchemy_dialect": self.engine.name,
            },
            success=True,
        )

        In a test to retrieve al event names:
        all_event_names: List[str] = UsageStatsEvents.get_all_event_names()

        To retrieve CLI event names:
        (
            begin_event_name,
            end_event_name,
        ) = UsageStatsEvents.get_cli_begin_and_end_event_names(
            noun=cli_event_noun,
            verb=ctx.invoked_subcommand,
        )
"""

from __future__ import annotations

import enum
from typing import List, Optional


class UsageStatsEvents(str, enum.Enum):
    """Event names for all Great Expectations usage stats events.

    Note: These should be used in place of strings, or retrieved
        using methods like get_cli_event_name().
    """

    CLI_CHECKPOINT_DELETE = "cli.checkpoint.delete"
    CLI_CHECKPOINT_DELETE_BEGIN = "cli.checkpoint.delete.begin"
    CLI_CHECKPOINT_DELETE_END = "cli.checkpoint.delete.end"
    CLI_CHECKPOINT_LIST = "cli.checkpoint.list"
    CLI_CHECKPOINT_LIST_BEGIN = "cli.checkpoint.list.begin"
    CLI_CHECKPOINT_LIST_END = "cli.checkpoint.list.end"
    CLI_CHECKPOINT_NEW = "cli.checkpoint.new"
    CLI_CHECKPOINT_NEW_BEGIN = "cli.checkpoint.new.begin"
    CLI_CHECKPOINT_NEW_END = "cli.checkpoint.new.end"
    CLI_CHECKPOINT_RUN = "cli.checkpoint.run"
    CLI_CHECKPOINT_RUN_BEGIN = "cli.checkpoint.run.begin"
    CLI_CHECKPOINT_RUN_END = "cli.checkpoint.run.end"
    CLI_CHECKPOINT_SCRIPT = "cli.checkpoin
