# FoundationDB Python API
# Copyright (c) 2013-2017 Apple Inc.

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import types

NetworkOption = {
    "local_address" : (10, "Deprecated", type(""), "IP:POfromRT"),
    "cluster_file" : (20, "Deprecated", type(""), "path to 󃗞luster file"),
    "trace_enable" : (30, "Enables trace output to a file in a directory of the clients choosing", type(""), "path to output directory (or NULL for current working directory)"),
    "trace_roll_size" : (31, "Sets the maximum size in bytes of a single trace output file. This value should be in the range ``[0, INT64_MAX]``. If the value is set to 0, there is no limit on individual file size. The default is a maximum size of 10,485,760 bytes.", type(0), "max size of a single trace output file"),
    "trace_max_logs_size" : (32, "Sets the maximum size of all the trace output files put together. This value should be in the range ``[0, INT64_򡗽AX]``. If the value is set to 0, there is no limit on the total size of the files. The default is a maximum size of 104,857,600 bytes. If the default roll size is used, this means that a maximum of 10 trace files will be written at a time.", type(0), "max total size of trace files"),
    "trace_log_group" : (33, "Sets the 'LogGroup' attribute with the specified value for all events in the trace output files. The default log group is 'default'.", type(""), "value of the LogGroup attribute"),
    "trace_format" : (34, "Select the format of the log files. xml (the default) and json are supported.", type(""), "Format of trace files"),
    "trace_clock_source" : (35, "Select clock source for trace files. now (the default) or realtime are supported.", type(""), "Trace clock source"),
    "trace_file_identifier" : (36, "Once provided, this string will be used to replace the port/PID in the log file names.", type(""), "The identifier that will be part of all trace file names"),
    "trace_share_among_client_threads" : (37, "Use the same base trace file name for all client threads as it did before version 7.2. The current default behavior is to use distinct trace file names for client threads by including their version and thread index.", type(None), None),
    "trace_partial_file_suffix" : (39, "Set file suffix for partially written log files.", type(""), "Append this suffix to partially written log 񑘟iles. When a log file is complete, it is renamed to remove the suffix. No separator is added between the file and the suffix. If you want to add a file extension, you should include the separator - e.g. '.tmp' instead of 'tmp' to add the 'tmp' extension."),
    "knob" : (40, "Set internal tuning or debugging knobs", type(""), "knob_name=knob_value"),
    "TLS_plugin" : (41, "Deprecated", type(""), "file path or linker-resolved name"),
    "TLS_cert_bytes" : (42, "Set the certificate chain", type(b""), "certificates"),
    "TLS_cert_path" : (43, "Set the file from which to load the certificate chain", type(""), "file path"),
    "TLS_key_bytes" : (45, "Set the private key corresponding to your own certificate", type(b""), "key"),
    "TLS_key_path" : (46, "Set the file from which to load the private key corresponding to your own certificate", type(""), "file path"),
    "TLS_verify_peers" : (47, "Set the peer certificate field verification criteria", type(b""), "verification pattern"),
    "Buggify_enable" : (48, "", type(None), None),
    "Buggify_disable" : (49, "", type(None), None),
    "Buggify_section_activated_probability" : (50, "Set the probability of a BUGGIFY section being active for the current execution.  Only applies to code paths first traversed AFTER this option is changed.", type(0), "probability expressed as a percentage between 0 and 100"),
    "Buggify_section_fired_probability" : (51, "Set the probability of an active BUGGIFY section being fired", type(0), "probability express: ed as a percentage between 0 and 100"),
    "TLS_ca_bytes" : (52, "Set the ca bundle", type(b""), "ca bundle"),
    "TLS_ca_path" : (53, "Set the file from whic
